"""
配置生成器

职责：生成完整、有效的 NodeConfig
- 这是唯一的配置生成入口
- CLI 和 PaperRegistry 都通过它生成配置
- 自动填充默认值、处理 partition 等

Usage:
    from oiafed.config import ConfigGenerator
    
    generator = ConfigGenerator()
    
    # 生成完整联邦配置（1 Trainer + N Learners）
    configs = generator.generate_federation(
        num_clients=10,
        trainer_args={"num_rounds": 100},
        learner_args={"learning_rate": 0.01},
    )
    
    # 或单独生成
    trainer = generator.generate_trainer(...)
    learner = generator.generate_learner(index=0, num_clients=10, ...)
    
    # 保存到文件
    generator.save_configs(configs, "./configs")
"""
from typing import Any, Dict, List, Optional, Union
from dataclasses import asdict
from pathlib import Path
import copy

from .defaults import (
    DEFAULT_TRAINER_PORT,
    DEFAULT_LEARNER_BASE_PORT,
    DEFAULT_HOST,
    DEFAULT_LOCALHOST,
    DEFAULT_MAX_MESSAGE_SIZE,
    DEFAULT_TIMEOUT,
    DEFAULT_TRAINER_TYPE,
    DEFAULT_LEARNER_TYPE,
    DEFAULT_AGGREGATOR_TYPE,
    DEFAULT_MODEL_TYPE,
    DEFAULT_DATASET_TYPE,
    DEFAULT_PARTITION_STRATEGY,
    DEFAULT_PARTITION_ALPHA,
    DEFAULT_LOG_LEVEL,
    DEFAULT_LOG_DIR,
    DEFAULT_SERIALIZATION,
    DEFAULT_TRANSPORT_MODE,
    DEFAULT_EXP_NAME,
)


class ConfigGenerator:
    """
    配置生成器
    
    统一的配置生成入口，确保生成的配置：
    1. 符合 NodeConfig Schema
    2. 包含所有必要的默认值
    3. 自动处理 partition_id, num_partitions 等
    
    设计原则：
    - 所有生成方法返回 NodeConfig 实例（不是字典）
    - 与 ConfigManager 协作：生成后可用 manager.save() 保存
    - 支持完全自定义，所有参数都可覆盖
    """
    
    def __init__(self):
        # 延迟导入避免循环依赖
        from .manager import ConfigManager
        self._manager = ConfigManager()
    
    # ==================== 内部方法 ====================
    
    def _generate_exp_name(
        self,
        model_type: str,
        aggregator_type: str,
        dataset_type: str,
        partition_strategy: str,
        partition_alpha: float,
        num_clients: int,
    ) -> str:
        """
        自动生成实验名称
        
        格式: {model}_{method}_{dataset}_{partition}{alpha}_n{num_clients}
        例如: cnn_fedavg_cifar10_dirichlet0.1_n10
        
        Args:
            model_type: 模型类型
            aggregator_type: 聚合器类型（作为方法名）
            dataset_type: 数据集类型
            partition_strategy: 分区策略
            partition_alpha: Dirichlet alpha
            num_clients: 客户端数量
            
        Returns:
            生成的实验名称
        """
        # 提取简短名称（去掉前缀如 "methods.models."）
        model = model_type.split(".")[-1]
        method = aggregator_type.split(".")[-1]
        dataset = dataset_type.split(".")[-1]
        
        # 构建分区部分
        if partition_strategy == "iid":
            partition_part = "iid"
        else:
            partition_part = f"{partition_strategy}{partition_alpha}"
        
        # 组合: model_method_dataset_partition_nX
        return f"{model}_{method}_{dataset}_{partition_part}_n{num_clients}"
    
    # ==================== 主要 API ====================
    
    def generate_federation(
        self,
        num_clients: int,
        *,
        # 全局配置
        exp_name: Optional[str] = None,  # None 表示自动生成
        run_name: Optional[str] = None,
        log_dir: str = DEFAULT_LOG_DIR,
        
        # 组件类型（可从 PaperDef 获取）
        trainer_type: str = DEFAULT_TRAINER_TYPE,
        learner_type: str = DEFAULT_LEARNER_TYPE,
        aggregator_type: str = DEFAULT_AGGREGATOR_TYPE,
        model_type: str = DEFAULT_MODEL_TYPE,
        dataset_type: str = DEFAULT_DATASET_TYPE,
        
        # 组件参数
        trainer_args: Optional[Dict[str, Any]] = None,
        learner_args: Optional[Dict[str, Any]] = None,
        aggregator_args: Optional[Dict[str, Any]] = None,
        model_args: Optional[Dict[str, Any]] = None,
        dataset_args: Optional[Dict[str, Any]] = None,
        
        # Trainer 的数据集配置（用于全局测试集评估）
        trainer_datasets: Optional[List[Dict[str, Any]]] = None,
        
        # 数据划分
        partition_strategy: str = DEFAULT_PARTITION_STRATEGY,
        partition_alpha: float = DEFAULT_PARTITION_ALPHA,
        partition_seed: Optional[int] = None,
        
        # 可选配置
        tracker: Optional[Dict[str, Any]] = None,
        callbacks: Optional[List[Dict[str, Any]]] = None,
        logging: Optional[Dict[str, Any]] = None,
        
        # 网络配置
        trainer_host: str = DEFAULT_LOCALHOST,
        trainer_port: int = DEFAULT_TRAINER_PORT,
        learner_host: str = DEFAULT_LOCALHOST,
        learner_base_port: int = DEFAULT_LEARNER_BASE_PORT,
        
        # 其他
        default_timeout: float = DEFAULT_TIMEOUT,
        transport_mode: str = DEFAULT_TRANSPORT_MODE,
        
    ) -> List:
        """
        生成完整的联邦学习配置
        
        Args:
            num_clients: 客户端数量
            exp_name: 实验名称（None 则自动生成 model_method_dataset_partition_nX 格式）
            run_name: 运行名称（可选，自动生成）
            log_dir: 日志目录
            trainer_type: Trainer 类型
            learner_type: Learner 类型
            aggregator_type: Aggregator 类型
            model_type: 模型类型
            dataset_type: 数据集类型
            trainer_args: Trainer 参数（num_rounds, local_epochs 等）
            learner_args: Learner 参数（learning_rate, batch_size 等）
            aggregator_args: Aggregator 参数
            model_args: 模型参数（num_classes 等）
            dataset_args: 数据集参数（data_dir, download 等）
            partition_strategy: 数据划分策略（dirichlet, iid, label 等）
            partition_alpha: Dirichlet alpha（越小越 non-IID）
            partition_seed: 划分随机种子
            tracker: 追踪配置（MLflow, WandB 等）
            callbacks: 回调配置列表
            logging: 日志配置
            trainer_host: Trainer 主机地址
            trainer_port: Trainer 端口
            learner_host: Learner 主机地址
            learner_base_port: Learner 基础端口（第 i 个 Learner 使用 base_port + i）
            default_timeout: 默认 RPC 超时时间
            transport_mode: 传输模式（grpc, memory）
            
        Returns:
            配置列表 [trainer_config, learner_0, learner_1, ..., learner_{n-1}]
        """
        # 自动生成 exp_name（如果未指定）
        if exp_name is None:
            exp_name = self._generate_exp_name(
                model_type=model_type,
                aggregator_type=aggregator_type,
                dataset_type=dataset_type,
                partition_strategy=partition_strategy,
                partition_alpha=partition_alpha,
                num_clients=num_clients,
            )
        
        configs = []
        
        # 生成 Trainer 配置（包含全局测试集）
        trainer_config = self.generate_trainer(
            exp_name=exp_name,
            run_name=run_name,
            log_dir=log_dir,
            trainer_type=trainer_type,
            aggregator_type=aggregator_type,
            model_type=model_type,
            trainer_args=trainer_args,
            aggregator_args=aggregator_args,
            model_args=model_args,
            # Trainer 的数据集配置（用于全局评估）
            datasets=trainer_datasets,  # 优先使用用户指定的
            dataset_type=dataset_type,
            dataset_args=dataset_args,
            tracker=tracker,
            callbacks=callbacks,
            logging=logging,
            host=trainer_host,
            port=trainer_port,
            min_peers=num_clients,
            default_timeout=default_timeout,
            transport_mode=transport_mode,
        )
        configs.append(trainer_config)
        
        # 生成 Learner 配置
        trainer_address = f"trainer@{trainer_host}:{trainer_port}"
        
        for i in range(num_clients):
            learner_config = self.generate_learner(
                index=i,
                num_clients=num_clients,
                exp_name=exp_name,
                run_name=run_name,
                log_dir=log_dir,
                learner_type=learner_type,
                model_type=model_type,
                dataset_type=dataset_type,
                learner_args=learner_args,
                model_args=model_args,
                dataset_args=dataset_args,
                partition_strategy=partition_strategy,
                partition_alpha=partition_alpha,
                partition_seed=partition_seed,
                tracker=tracker,
                callbacks=callbacks,
                logging=logging,
                connect_to=[trainer_address],
                host=learner_host,
                port=learner_base_port + i,
                default_timeout=default_timeout,
                transport_mode=transport_mode,
            )
            configs.append(learner_config)
        
        return configs
    
    def generate_trainer(
        self,
        *,
        node_id: str = "trainer",
        exp_name: str = DEFAULT_EXP_NAME,
        run_name: Optional[str] = None,
        log_dir: str = DEFAULT_LOG_DIR,
        trainer_type: str = DEFAULT_TRAINER_TYPE,
        aggregator_type: str = DEFAULT_AGGREGATOR_TYPE,
        model_type: str = DEFAULT_MODEL_TYPE,
        trainer_args: Optional[Dict[str, Any]] = None,
        aggregator_args: Optional[Dict[str, Any]] = None,
        model_args: Optional[Dict[str, Any]] = None,
        # 新增：Trainer 的数据集配置（用于全局测试集评估）
        datasets: Optional[List[Dict[str, Any]]] = None,
        dataset_type: str = DEFAULT_DATASET_TYPE,
        dataset_args: Optional[Dict[str, Any]] = None,
        tracker: Optional[Dict[str, Any]] = None,
        callbacks: Optional[List[Dict[str, Any]]] = None,
        logging: Optional[Dict[str, Any]] = None,
        host: str = DEFAULT_HOST,
        port: int = DEFAULT_TRAINER_PORT,
        min_peers: int = 1,
        default_timeout: float = DEFAULT_TIMEOUT,
        transport_mode: str = DEFAULT_TRANSPORT_MODE,
    ):
        """
        生成 Trainer 配置
        
        Args:
            node_id: 节点 ID
            exp_name: 实验名称
            run_name: 运行名称
            log_dir: 日志目录
            trainer_type: Trainer 类型
            aggregator_type: Aggregator 类型
            model_type: 模型类型
            trainer_args: Trainer 参数
            aggregator_args: Aggregator 参数
            model_args: 模型参数
            datasets: 数据集配置列表（用于全局测试集评估）
            dataset_type: 默认数据集类型（当 datasets=None 时使用）
            dataset_args: 默认数据集参数（当 datasets=None 时使用）
            tracker: 追踪配置
            callbacks: 回调配置
            logging: 日志配置
            host: 监听地址
            port: 监听端口
            min_peers: 最少等待的客户端数
            default_timeout: 默认超时
            transport_mode: 传输模式
            
        Returns:
            NodeConfig 实例
        """
        config_dict = {
            "node_id": node_id,
            "role": "trainer",
            "global_config": {
                "exp_name": exp_name,
                "run_name": run_name,
                "log_dir": log_dir,
            },
            "listen": {
                "host": host,
                "port": port,
            },
            "min_peers": min_peers,
            "default_timeout": default_timeout,
            "transport": {
                "mode": transport_mode,
                "grpc": {"max_message_size": DEFAULT_MAX_MESSAGE_SIZE},
            },
            "serialization": {"default": DEFAULT_SERIALIZATION},
            "trainer": {
                "type": trainer_type,
                "args": trainer_args or {},
            },
            "aggregator": {
                "type": aggregator_type,
                "args": aggregator_args or {},
            },
            "model": {
                "type": model_type,
                "args": model_args or {},
            },
        }
        
        # 添加数据集配置（用于全局测试集评估）
        if datasets is not None:
            # 使用用户提供的完整数据集配置
            config_dict["datasets"] = datasets
        elif dataset_type:
            # 自动生成测试集配置（用于 Trainer 端全局评估）
            config_dict["datasets"] = [
                {
                    "type": dataset_type,
                    "split": "test",
                    "args": dataset_args or {},  # 如果没有传递参数，使用空字典
                    # 注意：没有 partition 配置，表示使用完整测试集
                }
            ]
        
        # 添加可选配置
        self._add_optional_config(config_dict, tracker, callbacks, logging)
        
        return self._manager.from_dict(config_dict)
    
    def generate_learner(
        self,
        *,
        index: int,
        num_clients: int,
        node_id: Optional[str] = None,
        exp_name: str = DEFAULT_EXP_NAME,
        run_name: Optional[str] = None,
        log_dir: str = DEFAULT_LOG_DIR,
        learner_type: str = DEFAULT_LEARNER_TYPE,
        model_type: str = DEFAULT_MODEL_TYPE,
        dataset_type: str = DEFAULT_DATASET_TYPE,
        learner_args: Optional[Dict[str, Any]] = None,
        model_args: Optional[Dict[str, Any]] = None,
        dataset_args: Optional[Dict[str, Any]] = None,
        partition_strategy: str = DEFAULT_PARTITION_STRATEGY,
        partition_alpha: float = DEFAULT_PARTITION_ALPHA,
        partition_seed: Optional[int] = None,
        tracker: Optional[Dict[str, Any]] = None,
        callbacks: Optional[List[Dict[str, Any]]] = None,
        logging: Optional[Dict[str, Any]] = None,
        connect_to: Optional[List[str]] = None,
        host: str = DEFAULT_HOST,
        port: Optional[int] = None,
        default_timeout: float = DEFAULT_TIMEOUT,
        transport_mode: str = DEFAULT_TRANSPORT_MODE,
    ):
        """
        生成 Learner 配置
        
        Args:
            index: Learner 索引（0, 1, 2, ...）
            num_clients: 总客户端数（用于 partition）
            node_id: 节点 ID（默认 learner_{index}）
            exp_name: 实验名称
            run_name: 运行名称
            log_dir: 日志目录
            learner_type: Learner 类型
            model_type: 模型类型
            dataset_type: 数据集类型
            learner_args: Learner 参数
            model_args: 模型参数
            dataset_args: 数据集参数
            partition_strategy: 划分策略
            partition_alpha: Dirichlet alpha
            partition_seed: 随机种子
            tracker: 追踪配置
            callbacks: 回调配置
            logging: 日志配置
            connect_to: 连接目标
            host: 监听地址
            port: 监听端口（默认 base_port + index）
            default_timeout: 默认超时
            transport_mode: 传输模式
            
        Returns:
            NodeConfig 实例
        """
        # 默认值处理
        if node_id is None:
            node_id = f"learner_{index}"
        
        if connect_to is None:
            connect_to = [f"trainer@{DEFAULT_LOCALHOST}:{DEFAULT_TRAINER_PORT}"]
        
        if port is None:
            port = DEFAULT_LEARNER_BASE_PORT + index
        
        # 构建数据集配置
        datasets = self._build_datasets(
            dataset_type=dataset_type,
            dataset_args=dataset_args or {},
            partition_strategy=partition_strategy,
            partition_alpha=partition_alpha,
            partition_seed=partition_seed,
            num_partitions=num_clients,
            partition_id=index,
        )
        
        config_dict = {
            "node_id": node_id,
            "role": "learner",
            "global_config": {
                "exp_name": exp_name,
                "run_name": run_name,
                "log_dir": log_dir,
            },
            "listen": {
                "host": host,
                "port": port,
            },
            "connect_to": connect_to,
            "default_timeout": default_timeout,
            "transport": {
                "mode": transport_mode,
                "grpc": {"max_message_size": DEFAULT_MAX_MESSAGE_SIZE},
            },
            "serialization": {"default": DEFAULT_SERIALIZATION},
            "learner": {
                "type": learner_type,
                "args": learner_args or {},
            },
            "model": {
                "type": model_type,
                "args": model_args or {},
            },
            "datasets": datasets,
        }
        
        # 添加可选配置
        self._add_optional_config(config_dict, tracker, callbacks, logging)
        
        return self._manager.from_dict(config_dict)
    
    # ==================== 文件操作 ====================
    
    def save_configs(
        self,
        configs: List,
        output_dir: str,
    ) -> List[str]:
        """
        保存配置到文件
        
        Args:
            configs: NodeConfig 列表
            output_dir: 输出目录
            
        Returns:
            保存的文件路径列表
        """
        output_path = Path(output_dir)
        output_path.mkdir(parents=True, exist_ok=True)
        
        saved_files = []
        for config in configs:
            file_path = output_path / f"{config.node_id}.yaml"
            self._manager.save(config, str(file_path))
            saved_files.append(str(file_path))
        
        return saved_files
    
    # ==================== 辅助方法 ====================
    
    def _build_datasets(
        self,
        dataset_type: str,
        dataset_args: Dict[str, Any],
        partition_strategy: str,
        partition_alpha: float,
        partition_seed: Optional[int],
        num_partitions: int,
        partition_id: int,
    ) -> List[Dict[str, Any]]:
        """
        构建数据集配置列表（train + test）
        
        train 数据集带 partition 配置
        test 数据集不划分（共享）
        """
        # 构建 partition 配置
        partition_config = {
            "strategy": partition_strategy,
            "num_partitions": num_partitions,
            "partition_id": partition_id,
        }
        
        # 根据策略添加特定参数
        if partition_strategy == "dirichlet":
            partition_config["alpha"] = partition_alpha
        
        if partition_seed is not None:
            partition_config["seed"] = partition_seed
        
        # 训练集参数
        train_args = copy.deepcopy(dataset_args)
        
        # 测试集参数（排除 download）
        test_args = {k: v for k, v in dataset_args.items() if k != "download"}
        
        return [
            # 训练集（带划分）
            {
                "type": dataset_type,
                "split": "train",
                "args": train_args,
                "partition": partition_config,
            },
            # 测试集（不划分）
            {
                "type": dataset_type,
                "split": "test",
                "args": test_args,
            },
        ]
    
    def _add_optional_config(
        self,
        config_dict: Dict[str, Any],
        tracker: Optional[Dict[str, Any]],
        callbacks: Optional[List[Dict[str, Any]]],
        logging: Optional[Dict[str, Any]],
    ) -> None:
        """添加可选配置（仅当提供时）"""
        
        if tracker is not None:
            config_dict["tracker"] = tracker
        
        if callbacks is not None:
            config_dict["callbacks"] = callbacks
        
        if logging is not None:
            config_dict["logging"] = logging
        else:
            # 默认日志配置
            config_dict["logging"] = {
                "level": DEFAULT_LOG_LEVEL,
                "console": True,
            }
    
    # ==================== PaperRegistry 接口 ====================
    
    def from_paper_def(
        self,
        paper_defaults: Dict[str, Any],
        paper_components: Dict[str, str],
        num_clients: int,
        override: Optional[Dict[str, Any]] = None,
    ) -> List:
        """
        从论文定义生成配置
        
        这是 PaperRegistry 调用的入口点，将论文定义转换为配置。
        
        Args:
            paper_defaults: 论文默认配置
            paper_components: 论文组件类型映射 {"trainer": "...", "learner": "...", ...}
            num_clients: 客户端数量
            override: 用户覆盖配置
            
        Returns:
            NodeConfig 列表 [trainer, learner_0, learner_1, ...]
        """
        # 合并配置: paper_defaults + override
        merged = self._deep_merge(paper_defaults, override or {})
        
        # 提取参数
        params = self._extract_params_from_merged(merged, paper_components, num_clients)
        
        # 生成配置
        return self.generate_federation(**params)
    
    def _extract_params_from_merged(
        self,
        merged: Dict[str, Any],
        paper_components: Dict[str, str],
        num_clients: int,
    ) -> Dict[str, Any]:
        """从合并后的配置中提取 generate_federation 需要的参数"""
        
        # 获取 exp_name（如果是 "default" 或空，则使用 None 让 generate_federation 自动生成）
        exp_name = merged.get("global_config", {}).get("exp_name") or merged.get("global", {}).get("exp_name")
        if exp_name in (None, "", "default", DEFAULT_EXP_NAME):
            exp_name = None  # 让 generate_federation 自动生成
        
        params: Dict[str, Any] = {
            "num_clients": num_clients,
            
            # 全局配置
            "exp_name": exp_name,
            "run_name": merged.get("global_config", {}).get("run_name") or merged.get("global", {}).get("run_name"),
            "log_dir": merged.get("global_config", {}).get("log_dir") or merged.get("global", {}).get("log_dir") or DEFAULT_LOG_DIR,
            
            # 组件类型：优先用 paper_components，其次用 merged 中的 override
            "trainer_type": paper_components.get("trainer") or merged.get("trainer", {}).get("type") or DEFAULT_TRAINER_TYPE,
            "learner_type": paper_components.get("learner") or merged.get("learner", {}).get("type") or DEFAULT_LEARNER_TYPE,
            "aggregator_type": paper_components.get("aggregator") or merged.get("aggregator", {}).get("type") or DEFAULT_AGGREGATOR_TYPE,
            "model_type": paper_components.get("model") or merged.get("model", {}).get("type") or DEFAULT_MODEL_TYPE,
            "dataset_type": paper_components.get("dataset") or DEFAULT_DATASET_TYPE,
            
            # 组件参数
            "trainer_args": self._extract_component_args(merged.get("trainer", {})),
            "learner_args": self._extract_component_args(merged.get("learner", {})),
            "aggregator_args": self._extract_component_args(merged.get("aggregator", {})),
            "model_args": self._extract_component_args(merged.get("model", {})),
        }
        
        # 处理数据集配置
        self._extract_dataset_params(merged, params)
        
        # 可选配置
        if "tracker" in merged:
            params["tracker"] = merged["tracker"]
        if "callbacks" in merged:
            params["callbacks"] = merged["callbacks"]
        if "logging" in merged:
            params["logging"] = merged["logging"]
        if "default_timeout" in merged:
            params["default_timeout"] = merged["default_timeout"]
        if "transport" in merged:
            params["transport_mode"] = merged["transport"].get("mode", DEFAULT_TRANSPORT_MODE)
        
        return params
    
    def _extract_component_args(self, component_config: Dict[str, Any]) -> Dict[str, Any]:
        """提取组件参数（排除 type 字段）"""
        if isinstance(component_config, dict):
            # 如果有 args 字段，使用它
            if "args" in component_config:
                return copy.deepcopy(component_config["args"])
            # 否则排除 type 字段
            return {k: v for k, v in component_config.items() if k != "type"}
        return {}
    
    def _extract_dataset_params(
        self,
        merged: Dict[str, Any],
        params: Dict[str, Any],
    ) -> None:
        """提取数据集相关参数"""
        
        # 标准格式: datasets (复数)
        if "datasets" in merged:
            datasets_list = merged["datasets"]
            # 查找训练集配置
            train_ds = next(
                (ds for ds in datasets_list if ds.get("split") == "train"),
                datasets_list[0] if datasets_list else None
            )
            if train_ds:
                params["dataset_args"] = train_ds.get("args", {})
                params["dataset_type"] = train_ds.get("type", params.get("dataset_type", DEFAULT_DATASET_TYPE))
                
                partition = train_ds.get("partition", {})
                if partition:
                    params["partition_strategy"] = partition.get("strategy", DEFAULT_PARTITION_STRATEGY)
                    params["partition_alpha"] = partition.get("alpha", DEFAULT_PARTITION_ALPHA)
                    if "seed" in partition:
                        params["partition_seed"] = partition["seed"]
        
        # 简化格式: dataset (单数，向后兼容)
        elif "dataset" in merged:
            dataset_config = copy.deepcopy(merged["dataset"])
            partition = dataset_config.pop("partition", {})
            
            # dataset_type 可能在顶层
            if "type" in dataset_config:
                params["dataset_type"] = dataset_config.pop("type")
            
            params["dataset_args"] = dataset_config
            
            if partition:
                params["partition_strategy"] = partition.get("strategy", DEFAULT_PARTITION_STRATEGY)
                params["partition_alpha"] = partition.get("alpha", DEFAULT_PARTITION_ALPHA)
                if "seed" in partition:
                    params["partition_seed"] = partition["seed"]
    
    def _deep_merge(
        self,
        base: Dict[str, Any],
        override: Dict[str, Any],
    ) -> Dict[str, Any]:
        """深度合并字典"""
        result = copy.deepcopy(base)
        
        for key, value in override.items():
            if (
                key in result
                and isinstance(result[key], dict)
                and isinstance(value, dict)
            ):
                result[key] = self._deep_merge(result[key], value)
            else:
                result[key] = copy.deepcopy(value)
        
        return result


# ==================== 便捷函数 ====================

def generate_federation(num_clients: int, **kwargs) -> List:
    """
    生成联邦配置的便捷函数
    
    等同于 ConfigGenerator().generate_federation(num_clients, **kwargs)
    """
    return ConfigGenerator().generate_federation(num_clients, **kwargs)


__all__ = [
    "ConfigGenerator",
    "generate_federation",
]


"""
Node 通信层

轻量级、可扩展的节点通信框架，支持：
- 对等通信：任意节点可以调用任意节点的方法
- 多种传输模式：Memory（调试）、gRPC（生产）无感切换
- 可扩展性：通过 Interceptor 机制支持日志、认证、监控等横切关注点
- 类型安全：统一的消息格式和错误处理

Example:
    from oiafed.comm import Node
    from oiafed.config import NodeCommConfig
    
    async def handle_train(payload, ctx):
        return {"status": "completed"}
    
    config = NodeCommConfig(node_id="my_node")
    node = Node(config)
    node.register("train", handle_train)
    
    async with node:
        result = await node.call("other_node", "train", {"epochs": 10})

配置说明：
    所有配置类已统一到 oiafed.config 模块：
    from oiafed.config import (
        NodeCommConfig,       # 通信节点配置
        TransportConfig,      # 传输层配置
        GrpcConfig,           # gRPC 配置
        MemoryTransportConfig, # Memory 传输配置
        SerializationConfig,  # 序列化配置
        InterceptorConfig,    # 拦截器配置
        HeartbeatConfig,      # 心跳配置
        MethodOptions,        # 方法选项
    )
"""

from .node import Node

# 所有配置类从 oiafed.config 导入
from ..config import (
    # 传输配置
    TlsConfig,
    GrpcConfig,
    MemoryTransportConfig,
    TransportConfig,
    # 通信配置
    SerializationConfig,
    MethodSerializationConfig,
    InterceptorConfig,
    AuthConfig,
    RetryConfig,
    HeartbeatConfig,
    MethodOptions,
    # 节点配置
    NodeCommConfig,
    # 便捷函数
    load_config,
)

from .message import (
    Message,
    MessageType,
    MessageContext,
    ConnectionInfo,
    ConnectionStatus,
    ErrorInfo,
)
from .exceptions import (
    NodeError,
    NodeNotConnectedError,
    NodeDisconnectedError,
    CallTimeoutError,
    RemoteExecutionError,
    SerializationError,
    InterceptorAbort,
    AuthenticationError,
)
from .transport import (
    Transport,
    MemoryTransport,
    GrpcTransport,
    create_transport,
)
from .serialization import (
    Serializer,
    JsonSerializer,
    PickleSerializer,
    SerializerRegistry,
)
from .interceptor import (
    Interceptor,
    InterceptorChain,
    InterceptorContext,
    LoggingInterceptor,
    AuthInterceptor,
)

# 向后兼容别名
NodeConfig = NodeCommConfig  # 已废弃，使用 NodeCommConfig
GrpcTransportConfig = GrpcConfig  # 向后兼容

__version__ = "3.0.0"

__all__ = [
    # 核心
    "Node",
    
    # 传输配置（从 oiafed.config）
    "TlsConfig",
    "GrpcConfig",
    "GrpcTransportConfig",  # 向后兼容别名
    "MemoryTransportConfig",
    "TransportConfig",
    
    # 通信配置（从 oiafed.config）
    "SerializationConfig",
    "MethodSerializationConfig",
    "InterceptorConfig",
    "AuthConfig",
    "RetryConfig",
    "HeartbeatConfig",
    "MethodOptions",
    
    # 节点配置（从 oiafed.config）
    "NodeCommConfig",
    "NodeConfig",  # 向后兼容别名
    
    # 便捷函数
    "load_config",
    
    # 消息
    "Message",
    "MessageType",
    "MessageContext",
    "ConnectionInfo",
    "ConnectionStatus",
    "ErrorInfo",
    
    # 异常
    "NodeError",
    "NodeNotConnectedError",
    "NodeDisconnectedError",
    "CallTimeoutError",
    "RemoteExecutionError",
    "SerializationError",
    "InterceptorAbort",
    "AuthenticationError",
    
    # 传输层
    "Transport",
    "MemoryTransport",
    "GrpcTransport",
    "create_transport",
    
    # 序列化
    "Serializer",
    "JsonSerializer",
    "PickleSerializer",
    "SerializerRegistry",
    
    # 拦截器
    "Interceptor",
    "InterceptorChain",
    "InterceptorContext",
    "LoggingInterceptor",
    "AuthInterceptor",
]



"""
VFL (Vertical Federated Learning) 纵向联邦学习算法

实现的算法:
- SplitNN: Split Learning for Health (MIT Media Lab, 2018)
"""

from .splitnn import SplitNNLearner, SplitNNServerLearner, SplitModel

__all__ = [
    'SplitNNLearner',
    'SplitNNServerLearner', 
    'SplitModel',
]


"""
Fed-CPrompt: Contrastive Prompt for Rehearsal-Free Federated Continual Learning
CVPR 2023

Paper: https://openaccess.thecvf.com/content/CVPR2023/papers/Qi_CPFE_A_Query-Based_Federated_Learning_Framework_With_Prompt_Enhancement_CVPR_2023_paper.pdf

核心思想：
1. 使用提示学习(Prompt Learning)避免灾难性遗忘
2. 对比学习增强提示表示
3. 无需存储历史样本(Rehearsal-Free)
4. 基于预训练模型(如CLIP, ViT)的持续学习
"""

import torch
import torch.nn as nn
import torch.nn.functional as F
import torch.optim as optim
from torch.utils.data import DataLoader, Subset
from typing import Dict, Any, Optional, List
import copy
import numpy as np

from ....core.learner import Learner
from ....core.types import StepMetrics, EpochMetrics, TrainMetrics, EvalResult
from ....registry import learner


class PromptPool(nn.Module):
    """
    提示池模块

    管理多个任务的提示向量
    """

    def __init__(self, num_tasks: int, prompt_length: int, embed_dim: int):
        super().__init__()
        self.num_tasks = num_tasks
        self.prompt_length = prompt_length
        self.embed_dim = embed_dim

        # 每个任务的提示向量
        self.prompts = nn.Parameter(
            torch.randn(num_tasks, prompt_length, embed_dim)
        )

        # 提示选择键(用于自动选择提示)
        self.prompt_keys = nn.Parameter(
            torch.randn(num_tasks, embed_dim)
        )

    def forward(self, task_id: Optional[int] = None, query: Optional[torch.Tensor] = None):
        """
        获取任务特定的提示

        Args:
            task_id: 任务ID(如果已知)
            query: 查询向量(用于自动选择提示)

        Returns:
            提示向量
        """
        if task_id is not None:
            # Task-Incremental: 直接使用任务ID
            return self.prompts[task_id]

        elif query is not None:
            # Class-Incremental: 使用查询向量选择最相关的提示
            # 计算查询与所有提示键的相似度
            similarities = F.cosine_similarity(
                query.unsqueeze(1),  # [batch, 1, dim]
                self.prompt_keys.unsqueeze(0),  # [1, num_tasks, dim]
                dim=2
            )

            # 选择最相似的提示
            selected_indices = similarities.argmax(dim=1)

            # 返回选中的提示
            selected_prompts = self.prompts[selected_indices]
            return selected_prompts

        else:
            raise ValueError("Either task_id or query must be provided")


@learner('cl.fed_cprompt', description='Fed-CPrompt: Contrastive Prompt for Rehearsal-Free Federated Continual Learning (CVPR 2023)')
class FedCPromptLearner(Learner):
    """
    Fed-CPrompt 持续学习方法的Learner实现

    特点：
    - 基于提示学习的持续学习
    - 对比学习增强提示表示
    - 无需存储历史样本
    - 支持预训练模型微调
    """

    def __init__(
        self,
        model,
        datasets,
        tracker=None,
        callbacks=None,
        config=None,
        node_id=None
    ):
        """
        初始化 Fed-CPrompt Learner

        Args:
            model: 模型实例
            datasets: 数据集字典 {"train": [...], "test": [...]}
            tracker: 追踪器
            callbacks: 回调管理器
            config: 配置字典
            node_id: 节点ID
        """
        super().__init__(model, None, tracker, callbacks, config, node_id)

        # 保存数据集
        self._datasets = datasets or {}

        # 从配置提取参数
        self._lr = self._config.get('learning_rate', 0.001)
        self._bs = self._config.get('batch_size', 32)
        self._epochs = self._config.get('local_epochs', 5)
        self.optimizer_type = self._config.get('optimizer', 'Adam').upper()
        self.loss_type = self._config.get('loss', 'CrossEntropyLoss')

        # 持续学习参数
        self.num_tasks = self._config.get('num_tasks', 5)
        self.classes_per_task = self._config.get('classes_per_task', 2)
        self.scenario = self._config.get('scenario', 'class_incremental')

        # Fed-CPrompt特定参数
        self.prompt_length = self._config.get('prompt_length', 10)
        self.embed_dim = self._config.get('embed_dim', 768)
        self.use_prompt_pool = self._config.get('use_prompt_pool', True)
        self.contrastive_temperature = self._config.get('contrastive_temperature', 0.07)
        self.contrastive_weight = self._config.get('contrastive_weight', 0.5)
        self.freeze_backbone = self._config.get('freeze_backbone', True)

        self.device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')

        # 持续学习状态
        self.current_task_id = 0
        self.seen_classes = []

        # 提示池(延迟初始化)
        self.prompt_pool = None

        # 组件占位符
        self.torch_model = None
        self._optimizer = None
        self._criterion = None
        self._train_loader = None
        self._test_loader = None

        self.logger.info(
            f"FedCPromptLearner {node_id} initialized: "
            f"num_tasks={self.num_tasks}, classes_per_task={self.classes_per_task}, "
            f"prompt_length={self.prompt_length}"
        )

    async def setup(self, config: Dict) -> None:
        """
        初始化训练环境

        Args:
            config: 运行时配置
        """
        # 获取PyTorch模型
        self.torch_model = self._model.get_model()
        self.torch_model.to(self.device)

        # 如果需要冻结主干网络
        if self.freeze_backbone:
            self._freeze_backbone_params()

        # 初始化提示池(如果需要)
        if self.use_prompt_pool:
            self.prompt_pool = PromptPool(
                num_tasks=self.num_tasks,
                prompt_length=self.prompt_length,
                embed_dim=self.embed_dim
            ).to(self.device)
            self.logger.info(f"Prompt pool initialized: {self.num_tasks} tasks")

        # 创建优化器
        # 收集需要优化的参数(只优化未冻结的参数和提示池)
        params_to_optimize = [p for p in self.torch_model.parameters() if p.requires_grad]

        # 如果有提示池，也加入优化
        if self.prompt_pool is not None:
            params_to_optimize += list(self.prompt_pool.parameters())

        if self.optimizer_type == 'SGD':
            self._optimizer = optim.SGD(
                params_to_optimize,
                lr=self._lr,
                momentum=self._config.get('momentum', 0.9),
                weight_decay=self._config.get('weight_decay', 5e-4)
            )
        elif self.optimizer_type == 'ADAM':
            self._optimizer = optim.Adam(
                params_to_optimize,
                lr=self._lr
            )
        else:
            raise ValueError(f"不支持的优化器类型: {self.optimizer_type}")

        # 创建损失函数
        if self.loss_type == 'CrossEntropyLoss':
            self._criterion = nn.CrossEntropyLoss()
        elif self.loss_type == 'MSELoss':
            self._criterion = nn.MSELoss()
        else:
            raise ValueError(f"不支持的损失函数: {self.loss_type}")

        # 创建数据加载器
        train_datasets = self._datasets.get("train", [])
        if train_datasets:
            train_dataset = train_datasets[0]
            self._train_loader = DataLoader(
                train_dataset,
                batch_size=self._bs,
                shuffle=True,
                drop_last=False
            )
            self.logger.info(
                f"Setup completed: train_samples={len(train_dataset)}, "
                f"batch_size={self._bs}"
            )

        test_datasets = self._datasets.get("test", [])
        if test_datasets:
            test_dataset = test_datasets[0]
            self._test_loader = DataLoader(
                test_dataset,
                batch_size=self._bs,
                shuffle=False
            )

        # 从配置中获取任务ID（如果有）
        task_id = config.get('task_id', self.current_task_id)
        if task_id != self.current_task_id:
            self._update_task_state(task_id)

    def _freeze_backbone_params(self):
        """冻结主干网络参数，只训练分类头"""
        # 假设模型的最后一层是分类器，冻结其他层
        for name, param in self.torch_model.named_parameters():
            # 通常分类器层包含'fc', 'classifier', 'head'等关键词
            if not any(keyword in name.lower() for keyword in ['fc', 'classifier', 'head']):
                param.requires_grad = False
                self.logger.debug(f"Frozen parameter: {name}")

    def _update_task_state(self, task_id: int):
        """
        更新任务状态

        Args:
            task_id: 新任务ID
        """
        self.logger.info(
            f"[{self._node_id}] Switching task: {self.current_task_id} -> {task_id}"
        )
        self.current_task_id = task_id

        # 更新已见类别
        start_class = task_id * self.classes_per_task
        end_class = min(start_class + self.classes_per_task,
                       self.num_tasks * self.classes_per_task)
        new_classes = list(range(start_class, end_class))

        # 只添加新的类别
        for cls in new_classes:
            if cls not in self.seen_classes:
                self.seen_classes.append(cls)

    def _get_task_data_loader(self, task_id: int, dataset=None) -> DataLoader:
        """
        获取特定任务的数据加载器

        根据当前任务ID筛选对应的类别数据

        Args:
            task_id: 任务ID
            dataset: 数据集（如果不提供则使用训练集）

        Returns:
            DataLoader: 任务数据加载器
        """
        if dataset is None:
            train_datasets = self._datasets.get("train", [])
            if not train_datasets:
                self.logger.warning("No train dataset available")
                return self._train_loader
            dataset = train_datasets[0]

        # 计算当前任务的类别范围
        start_class = task_id * self.classes_per_task
        end_class = min(start_class + self.classes_per_task,
                       self.num_tasks * self.classes_per_task)
        task_classes = list(range(start_class, end_class))

        # 筛选当前任务的样本
        indices = []
        for idx in range(len(dataset)):
            _, label = dataset[idx]
            if isinstance(label, torch.Tensor):
                label = label.item()
            if label in task_classes:
                indices.append(idx)

        if not indices:
            self.logger.warning(
                f"Task {task_id} has no samples for classes {task_classes}"
            )
            return self._train_loader

        task_dataset = Subset(dataset, indices)

        self.logger.info(
            f"Task {task_id} data loader: {len(indices)} samples, "
            f"classes {task_classes}"
        )

        return DataLoader(
            task_dataset,
            batch_size=self._bs,
            shuffle=True,
            drop_last=False
        )

    def get_dataloader(self) -> DataLoader:
        """
        获取数据加载器（Learner基类要求）

        Returns:
            DataLoader: 当前任务的数据加载器
        """
        return self._get_task_data_loader(self.current_task_id)

    def get_num_samples(self) -> int:
        """
        获取训练样本数

        Returns:
            int: 样本数量
        """
        train_datasets = self._datasets.get("train", [])
        if train_datasets:
            return len(train_datasets[0])
        return 0

    async def train_step(self, batch: Any, batch_idx: int) -> StepMetrics:
        """
        单批次训练

        Fed-CPrompt的训练流程：
        1. 使用任务特定的提示
        2. 通过对比学习增强提示表示
        3. 冻结主干网络，只更新提示和分类头

        Args:
            batch: 批次数据 (data, target)
            batch_idx: 批次索引

        Returns:
            StepMetrics: 训练指标
        """
        data, target = batch
        data, target = data.to(self.device), target.to(self.device)

        self._optimizer.zero_grad()

        # 获取提示增强的特征和输出
        features, output = self._forward_with_prompt(data, self.current_task_id)

        # 1. 分类损失
        ce_loss = self._criterion(output, target)

        # 2. 对比学习损失(增强提示表示)
        contrastive_loss = 0.0
        if self.use_prompt_pool and self.current_task_id > 0:
            contrastive_loss = self._compute_contrastive_loss(features, target, self.current_task_id)

        # 3. 提示正则化损失(防止提示偏移过大)
        prompt_reg_loss = 0.0
        if self.use_prompt_pool:
            prompt_reg_loss = self._compute_prompt_regularization()

        # 总损失
        if self.current_task_id > 0:
            loss = (ce_loss +
                    self.contrastive_weight * contrastive_loss +
                    0.01 * prompt_reg_loss)
        else:
            loss = ce_loss + 0.01 * prompt_reg_loss

        # 反向传播
        loss.backward()
        self._optimizer.step()

        # 计算准确率
        _, predicted = output.max(1)
        correct = predicted.eq(target).sum().item()
        accuracy = correct / target.size(0)

        return StepMetrics(
            loss=loss.item(),
            batch_size=target.size(0),
            metrics={
                'accuracy': accuracy,
                'ce_loss': ce_loss.item(),
                'contrastive_loss': contrastive_loss.item() if isinstance(contrastive_loss, torch.Tensor) else contrastive_loss,
                'prompt_reg_loss': prompt_reg_loss.item() if isinstance(prompt_reg_loss, torch.Tensor) else prompt_reg_loss
            }
        )

    async def train_epoch(self, epoch_idx: int) -> EpochMetrics:
        """
        单轮训练

        Args:
            epoch_idx: 轮次索引

        Returns:
            EpochMetrics: 轮次指标
        """
        # 调用父类的train_epoch方法
        self.torch_model.train()
        if self.prompt_pool is not None:
            self.prompt_pool.train()

        epoch_metrics = await super().train_epoch(epoch_idx)

        return epoch_metrics

    async def fit(self, config: Optional[Dict[str, Any]] = None) -> TrainMetrics:
        """
        执行完整的训练流程

        Args:
            config: 训练配置

        Returns:
            TrainMetrics: 训练指标
        """
        config = config or {}

        # 提取任务ID并更新任务状态
        task_id = config.get("task_id", self.current_task_id)
        if task_id != self.current_task_id:
            self._update_task_state(task_id)

        # 调用父类的fit方法
        return await super().fit(config)

    def _forward_with_prompt(self, data: torch.Tensor, task_id: int):
        """
        前向传播并应用提示

        Args:
            data: 输入数据
            task_id: 任务ID

        Returns:
            (features, output): 特征和输出
        """
        # 简化实现：直接使用模型输出
        # 实际应该在模型内部注入提示向量
        output = self.torch_model(data)

        # 使用输出作为特征(简化版)
        features = output.detach()

        return features, output

    def _compute_contrastive_loss(
        self,
        features: torch.Tensor,
        labels: torch.Tensor,
        task_id: int
    ) -> torch.Tensor:
        """
        计算对比学习损失

        增强同类样本的提示表示相似度，降低不同类样本的相似度

        Args:
            features: 特征向量
            labels: 标签
            task_id: 任务ID

        Returns:
            对比损失
        """
        if not self.use_prompt_pool:
            return torch.tensor(0.0, device=self.device)

        batch_size = features.size(0)

        # 归一化特征
        features_norm = F.normalize(features, p=2, dim=1)

        # 计算相似度矩阵
        similarity_matrix = torch.mm(features_norm, features_norm.t())

        # 创建标签掩码(同类为1，不同类为0)
        labels_expanded = labels.unsqueeze(0)
        mask = (labels_expanded == labels_expanded.t()).float()

        # 对角线设为0(排除自身)
        mask = mask - torch.eye(batch_size, device=self.device)

        # 正样本对和负样本对
        pos_mask = mask
        neg_mask = 1 - mask - torch.eye(batch_size, device=self.device)

        # 使用InfoNCE损失
        # exp(sim(i,j)/temp) / sum(exp(sim(i,k)/temp))
        exp_sim = torch.exp(similarity_matrix / self.contrastive_temperature)

        # 对于每个样本，计算正样本对的损失
        pos_sum = (exp_sim * pos_mask).sum(dim=1)
        neg_sum = (exp_sim * neg_mask).sum(dim=1)

        # 避免除零
        loss = -torch.log((pos_sum + 1e-8) / (pos_sum + neg_sum + 1e-8))

        return loss.mean()

    def _compute_prompt_regularization(self) -> torch.Tensor:
        """
        计算提示正则化损失

        防止提示向量偏离初始化太远

        Returns:
            正则化损失
        """
        if not self.use_prompt_pool:
            return torch.tensor(0.0, device=self.device)

        # L2正则化
        reg_loss = 0.0
        for param in self.prompt_pool.parameters():
            reg_loss += (param ** 2).sum()

        return reg_loss

    async def evaluate(self, config: Optional[Dict[str, Any]] = None) -> EvalResult:
        """
        评估模型性能

        Args:
            config: 评估配置

        Returns:
            EvalResult: 评估结果
        """
        config = config or {}
        task_id = config.get("task_id", self.current_task_id)

        # 获取评估数据
        if self._test_loader is not None:
            eval_loader = self._test_loader
        else:
            # 使用训练集评估
            if task_id is not None:
                eval_loader = self._get_task_data_loader(task_id)
            else:
                eval_loader = self._train_loader

        self.torch_model.eval()
        if self.prompt_pool is not None:
            self.prompt_pool.eval()

        total_loss = 0.0
        correct = 0
        total = 0

        with torch.no_grad():
            for data, target in eval_loader:
                data, target = data.to(self.device), target.to(self.device)

                # 前向传播
                _, output = self._forward_with_prompt(data, task_id)

                loss = self._criterion(output, target)

                total_loss += loss.item()
                _, predicted = output.max(1)
                total += target.size(0)
                correct += predicted.eq(target).sum().item()

        accuracy = correct / total if total > 0 else 0
        avg_loss = total_loss / len(eval_loader) if len(eval_loader) > 0 else 0

        return EvalResult(
            num_samples=total,
            metrics={
                'accuracy': accuracy,
                'loss': avg_loss,
                'task_id': task_id
            },
            metadata={
                'scenario': self.scenario,
                'use_prompt_pool': self.use_prompt_pool
            }
        )

    def get_prompt_summary(self) -> Dict[str, Any]:
        """
        获取提示摘要

        Returns:
            提示统计信息
        """
        return {
            'use_prompt_pool': self.use_prompt_pool,
            'prompt_length': self.prompt_length,
            'embed_dim': self.embed_dim,
            'current_task': self.current_task_id,
            'seen_classes': self.seen_classes
        }

    def get_metadata(self) -> Dict[str, Any]:
        """
        获取训练元数据

        Returns:
            元数据字典
        """
        base_metadata = super().get_metadata()
        base_metadata.update({
            'task_id': self.current_task_id,
            'seen_classes': len(self.seen_classes),
            'prompt_summary': self.get_prompt_summary()
        })
        return base_metadata



"""
联邦学习核心组件
"""

from .types import (
    TrainResult,
    EvalResult,
    ClientUpdate,
    ClientInfo,
    FitConfig,
    EvalConfig,
    FitStatus,
)
from .learner import Learner
from .trainer import Trainer
from .aggregator import Aggregator
from .system import FederatedSystem

# scenario_trainers 和 scenario_learners 已删除
# 使用显式 callback 调用替代 __init_subclass__ 自动装饰器机制

__all__ = [
    # 类型
    "TrainResult",
    "EvalResult",
    "ClientUpdate",
    "ClientInfo",
    "FitConfig",
    "EvalConfig",
    "FitStatus",

    # 抽象基类
    "Learner",
    "Trainer",
    "Aggregator",

    # 系统容器
    "FederatedSystem",
]



"""
DataProvider - 数据提供者接口

定义数据访问的统一接口
"""

from abc import ABC, abstractmethod
from typing import Any, Dict, Iterator, List, Optional, Tuple


class DataProvider(ABC):
    """
    数据提供者抽象基类
    
    职责：
    - 提供训练数据迭代器
    - 提供评估数据迭代器
    - 提供数据信息
    """
    
    @abstractmethod
    def get_train_loader(self) -> Iterator[Tuple[Any, Any]]:
        """
        获取训练数据加载器
        
        Returns:
            迭代器，每次返回 (batch_x, batch_y)
        """
        pass
    
    @abstractmethod
    def get_eval_loader(self) -> Iterator[Tuple[Any, Any]]:
        """
        获取评估数据加载器
        
        Returns:
            迭代器，每次返回 (batch_x, batch_y)
        """
        pass
    
    @abstractmethod
    def get_info(self) -> Dict[str, Any]:
        """
        获取数据信息
        
        Returns:
            数据信息字典，至少包含 num_samples
        """
        pass
    
    def get_num_samples(self) -> int:
        """获取样本数量"""
        return self.get_info().get("num_samples", 0)


class InMemoryDataProvider(DataProvider):
    """
    内存数据提供者
    
    简单的内存数据提供者，用于测试或小数据集
    """
    
    def __init__(
        self,
        train_x: Any,
        train_y: Any,
        eval_x: Optional[Any] = None,
        eval_y: Optional[Any] = None,
        batch_size: int = 32,
    ):
        """
        初始化
        
        Args:
            train_x: 训练特征
            train_y: 训练标签
            eval_x: 评估特征（可选）
            eval_y: 评估标签（可选）
            batch_size: 批大小
        """
        self._train_x = train_x
        self._train_y = train_y
        self._eval_x = eval_x if eval_x is not None else train_x
        self._eval_y = eval_y if eval_y is not None else train_y
        self._batch_size = batch_size
    
    def get_train_loader(self) -> Iterator[Tuple[Any, Any]]:
        """获取训练数据加载器"""
        return self._batch_iterator(self._train_x, self._train_y)
    
    def get_eval_loader(self) -> Iterator[Tuple[Any, Any]]:
        """获取评估数据加载器"""
        return self._batch_iterator(self._eval_x, self._eval_y)
    
    def get_info(self) -> Dict[str, Any]:
        """获取数据信息"""
        return {
            "num_samples": len(self._train_x),
            "num_eval_samples": len(self._eval_x),
            "batch_size": self._batch_size,
        }
    
    def _batch_iterator(self, x: Any, y: Any) -> Iterator[Tuple[Any, Any]]:
        """批次迭代器"""
        n = len(x)
        for i in range(0, n, self._batch_size):
            end = min(i + self._batch_size, n)
            yield x[i:end], y[i:end]


class SubsetDataProvider(DataProvider):
    """
    子集数据提供者
    
    从完整数据集中按 indices 选择子集
    """
    
    def __init__(
        self,
        full_provider: DataProvider,
        indices: List[int],
        batch_size: int = 32,
    ):
        """
        初始化
        
        Args:
            full_provider: 完整数据提供者
            indices: 选择的索引列表
            batch_size: 批大小
        """
        self._full_provider = full_provider
        self._indices = indices
        self._batch_size = batch_size
        
        # 提取子集（这里假设 full_provider 支持索引访问）
        self._subset_loaded = False
        self._train_x = None
        self._train_y = None
    
    def _ensure_loaded(self) -> None:
        """确保数据已加载"""
        if self._subset_loaded:
            return
        
        # 尝试从 full_provider 获取数据
        info = self._full_provider.get_info()
        if "data" in info:
            full_x, full_y = info["data"]
            self._train_x = [full_x[i] for i in self._indices]
            self._train_y = [full_y[i] for i in self._indices]
            self._subset_loaded = True
    
    def get_train_loader(self) -> Iterator[Tuple[Any, Any]]:
        """获取训练数据加载器"""
        self._ensure_loaded()
        if self._train_x is None:
            return iter([])
        
        n = len(self._train_x)
        for i in range(0, n, self._batch_size):
            end = min(i + self._batch_size, n)
            yield self._train_x[i:end], self._train_y[i:end]
    
    def get_eval_loader(self) -> Iterator[Tuple[Any, Any]]:
        """获取评估数据加载器"""
        return self.get_train_loader()
    
    def get_info(self) -> Dict[str, Any]:
        """获取数据信息"""
        return {
            "num_samples": len(self._indices),
            "indices": self._indices,
            "batch_size": self._batch_size,
        }



"""
论文注册表模块

提供论文定义的加载、查询和配置生成功能。

使用方式:
    from src.papers import get_registry
    
    registry = get_registry()
    
    # 列出所有论文
    papers = registry.list_all()
    
    # 获取论文详情
    paper = registry.get("fot")
    
    # 生成配置
    configs = registry.generate_node_configs("fot", override={"learner": {"lr": 0.001}})
"""

from .loader import (
    # 数据类
    ParamDef,
    PaperDef,
    
    # 注册表
    PaperRegistry,
    get_registry,
    reload_registry,
)


__all__ = [
    "ParamDef",
    "PaperDef",
    "PaperRegistry",
    "get_registry",
    "reload_registry",
]



"""
配置系统 v4.0

葡萄结构：每个配置类与其解析方法在同一模块中，自包含。

结构：
    config/
    ├── transport.py    # 传输层配置
    ├── comm.py         # 通信层配置
    ├── logging_config.py # 日志配置
    ├── tracker.py      # 追踪配置
    ├── component.py    # 组件配置
    ├── node.py         # 节点配置（根）
    ├── manager.py      # 高层管理器
    ├── generator.py    # 配置生成
    └── defaults.py     # 默认值常量

配置树：
    NodeConfig (根)
    ├── GlobalConfig (共享字段源: exp_name, run_name, log_dir)
    ├── LogConfig
    ├── TrackerConfig
    │   └── TrackerBackendConfig
    ├── TransportConfig
    │   ├── GrpcConfig
    │   │   └── TlsConfig
    │   └── MemoryTransportConfig
    ├── SerializationConfig
    ├── InterceptorConfig
    ├── HeartbeatConfig
    ├── ConnectionRetryConfig
    ├── ComponentConfig (trainer/learner/aggregator/model)
    ├── DatasetConfig[]
    └── CallbackConfig[]

Quick Start:
    from oiafed.config import load_config, NodeConfig
    
    # 从文件加载
    config = load_config("configs/trainer.yaml")
    
    # 访问配置
    print(config.node_id)
    print(config.exp_name)  # 从 GlobalConfig 同步
    print(config.transport.grpc.max_message_size)
    
    # 获取通信配置
    comm_config = config.get_comm_config()
"""

# ==================== 传输层配置 ====================

from .transport import (
    # 配置类
    TlsConfig,
    GrpcConfig,
    MemoryTransportConfig,
    TransportConfig,
    ConnectionRetryConfig,
    # 解析方法
    parse_tls_config,
    parse_grpc_config,
    parse_memory_config,
    parse_transport_config,
    parse_connection_retry_config,
)

# ==================== 通信层配置 ====================

from .comm import (
    # 序列化
    MethodSerializationConfig,
    SerializationConfig,
    parse_method_serialization_config,
    parse_serialization_config,
    # 认证
    AuthConfig,
    parse_auth_config,
    # 重试
    RetryConfig,
    parse_retry_config,
    # 拦截器
    InterceptorConfig,
    parse_interceptor_config,
    # 心跳
    HeartbeatConfig,
    parse_heartbeat_config,
    # 方法选项
    MethodOptions,
)

# ==================== 日志配置 ====================

from .logging_config import (
    LogConfig,
    parse_log_config,
)

# ==================== 追踪配置 ====================

from .tracker import (
    MLflowConfig,
    parse_mlflow_config,
    WandbConfig,
    parse_wandb_config,
    TensorBoardConfig,
    parse_tensorboard_config,
    TrackerBackendConfig,
    parse_tracker_backend_config,
    TrackerConfig,
    parse_tracker_config,
)

# ==================== 组件配置 ====================

from .component import (
    ComponentConfig,
    parse_component_config,
    DatasetConfig,
    parse_dataset_config,
    parse_datasets_config,
    CallbackConfig,
    parse_callback_config,
    parse_callbacks_config,
)

# ==================== 节点配置（根）====================

from .node import (
    # 枚举
    LogLevel,
    TransportMode,
    BackoffStrategy,
    NodeRole,
    # 全局配置
    GlobalConfig,
    parse_global_config,
    # 通信节点配置
    NodeCommConfig,
    # 节点配置
    NodeConfig,
    parse_node_config,
)

# ==================== 管理器 ====================

from .manager import (
    # 异常
    ConfigError,
    ConfigValidationError,
    ConfigLoadError,
    # 管理器
    ConfigManager,
    get_default_manager,
    # 便捷函数
    load_config,
    load_config_from_dict,
    save_config,
    validate_config,
    config_to_dict,
    # 向后兼容
    load_node_config,
    deep_merge,
    create_client_config,
)

# ==================== 配置生成器 ====================

from .generator import (
    ConfigGenerator,
    generate_federation,
)

# ==================== 默认值 ====================

from .defaults import (
    DEFAULT_TRAINER_PORT,
    DEFAULT_LEARNER_BASE_PORT,
    DEFAULT_HOST,
    DEFAULT_LOCALHOST,
    DEFAULT_MAX_MESSAGE_SIZE,
    DEFAULT_TIMEOUT,
    DEFAULT_TRAINER_TYPE,
    DEFAULT_LEARNER_TYPE,
    DEFAULT_AGGREGATOR_TYPE,
    DEFAULT_MODEL_TYPE,
    DEFAULT_DATASET_TYPE,
    DEFAULT_PARTITION_STRATEGY,
    DEFAULT_PARTITION_ALPHA,
    DEFAULT_LOG_LEVEL,
    DEFAULT_LOG_DIR,
    DEFAULT_SERIALIZATION,
    DEFAULT_TRANSPORT_MODE,
    DEFAULT_EXP_NAME,
)

# ==================== 向后兼容别名 ====================

# 旧名称映射
GrpcTransportConfig = GrpcConfig  # 向后兼容
FederationConfig = NodeConfig
LoggingConfig = LogConfig

# ==================== 版本信息 ====================

__version__ = "4.0.0"

# ==================== 导出 ====================

__all__ = [
    # 版本
    "__version__",
    
    # 枚举
    "LogLevel",
    "TransportMode",
    "BackoffStrategy",
    "NodeRole",
    
    # 传输配置
    "TlsConfig",
    "GrpcConfig",
    "GrpcTransportConfig",  # 向后兼容
    "MemoryTransportConfig",
    "TransportConfig",
    "ConnectionRetryConfig",
    "parse_tls_config",
    "parse_grpc_config",
    "parse_memory_config",
    "parse_transport_config",
    "parse_connection_retry_config",
    
    # 通信配置
    "MethodSerializationConfig",
    "SerializationConfig",
    "AuthConfig",
    "RetryConfig",
    "InterceptorConfig",
    "HeartbeatConfig",
    "MethodOptions",
    "parse_method_serialization_config",
    "parse_serialization_config",
    "parse_auth_config",
    "parse_retry_config",
    "parse_interceptor_config",
    "parse_heartbeat_config",
    
    # 日志配置
    "LogConfig",
    "parse_log_config",
    
    # 追踪配置
    "MLflowConfig",
    "WandbConfig",
    "TensorBoardConfig",
    "TrackerBackendConfig",
    "TrackerConfig",
    "parse_mlflow_config",
    "parse_wandb_config",
    "parse_tensorboard_config",
    "parse_tracker_backend_config",
    "parse_tracker_config",
    
    # 组件配置
    "ComponentConfig",
    "DatasetConfig",
    "CallbackConfig",
    "parse_component_config",
    "parse_dataset_config",
    "parse_datasets_config",
    "parse_callback_config",
    "parse_callbacks_config",
    
    # 全局配置
    "GlobalConfig",
    "parse_global_config",
    
    # 通信节点配置
    "NodeCommConfig",
    
    # 节点配置（根）
    "NodeConfig",
    "parse_node_config",
    
    # 异常
    "ConfigError",
    "ConfigValidationError",
    "ConfigLoadError",
    
    # 管理器
    "ConfigManager",
    "get_default_manager",
    
    # 生成器
    "ConfigGenerator",
    "generate_federation",
    
    # 便捷函数
    "load_config",
    "load_config_from_dict",
    "save_config",
    "validate_config",
    "config_to_dict",
    
    # 默认值
    "DEFAULT_TRAINER_PORT",
    "DEFAULT_LEARNER_BASE_PORT",
    "DEFAULT_HOST",
    "DEFAULT_LOCALHOST",
    "DEFAULT_MAX_MESSAGE_SIZE",
    "DEFAULT_TIMEOUT",
    "DEFAULT_TRAINER_TYPE",
    "DEFAULT_LEARNER_TYPE",
    "DEFAULT_AGGREGATOR_TYPE",
    "DEFAULT_MODEL_TYPE",
    "DEFAULT_DATASET_TYPE",
    "DEFAULT_PARTITION_STRATEGY",
    "DEFAULT_PARTITION_ALPHA",
    "DEFAULT_LOG_LEVEL",
    "DEFAULT_LOG_DIR",
    "DEFAULT_SERIALIZATION",
    "DEFAULT_TRANSPORT_MODE",
    "DEFAULT_EXP_NAME",
    
    # 向后兼容
    "load_node_config",
    "deep_merge",
    "create_client_config",
    "FederationConfig",
    "LoggingConfig",
]



"""
FedCP (Federated Contrastive Personalization) 学习器实现

从 methods/learners/fl/fedcp.py 迁移到新架构

FedCP结合了对比学习和个性化联邦学习

核心思想：
- 使用对比学习来学习更好的特征表示
- 分离共享特征提取器和个性化分类头
- 通过对比损失拉近同类样本，推远不同类样本
"""
from typing import Dict, Any, Optional
import torch
import torch.nn as nn
import torch.nn.functional as F
import torch.optim as optim
from torch.utils.data import DataLoader

from ....core.learner import Learner
from ....core.types import StepMetrics, EpochMetrics, EvalResult
from ....registry import learner


@learner('fl.fedcp', description='FedCP: Federated Contrastive Personalization')
class FedCPLearner(Learner):
    """FedCP学习器 - 对比个性化联邦学习

    配置示例:
    {
        "learner": {
            "name": "fl.fedcp",
            "learning_rate": 0.01,
            "batch_size": 128,
            "local_epochs": 5,
            "optimizer": "SGD",
            "momentum": 0.9,
            "head_layer_names": ["fc2"],  # 个性化头部层的名称
            "lambda_contrast": 0.1,  # 对比损失权重
            "temperature": 0.5  # 对比学习温度
        }
    }
    """

    def __init__(
        self,
        model,
        datasets,
        tracker=None,
        callbacks=None,
        config=None,
        node_id=None
    ):
        super().__init__(model, None, tracker, callbacks, config, node_id)

        self._datasets = datasets or {}

        # 从配置提取参数
        self.learning_rate = self._config.get('learning_rate', 0.01)
        self.batch_size = self._config.get('batch_size', 32)
        self.local_epochs = self._config.get('local_epochs', 5)
        self.momentum = self._config.get('momentum', 0.9)
        self.optimizer_type = self._config.get('optimizer', 'SGD').upper()

        # FedCP特有参数
        self.head_layer_names = self._config.get(
            'head_layer_names',
            ['fc2', 'fc', 'classifier', 'head']
        )
        self.lambda_contrast = self._config.get('lambda_contrast', 0.1)
        self.temperature = self._config.get('temperature', 0.5)

        self.device = torch.device('cuda' if torch.cuda.is_available() else 'cpu')

        # 组件占位符
        self._optimizer = None
        self._criterion = None
        self._train_loader = None
        self._test_loader = None

        self.logger.info(
            f"FedCPLearner {node_id} 初始化完成 "
            f"(lr={self.learning_rate}, lambda_contrast={self.lambda_contrast})"
        )

    async def setup(self, config: Dict) -> None:
        """初始化训练环境"""
        # 获取PyTorch模型
        self.model.to(self.device)

        # 创建优化器
        if self.optimizer_type == 'SGD':
            self._optimizer = optim.SGD(
                self.model.parameters(),
                lr=self.learning_rate,
                momentum=self.momentum
            )
        elif self.optimizer_type == 'ADAM':
            self._optimizer = optim.Adam(
                self.model.parameters(),
                lr=self.learning_rate
            )
        else:
            raise ValueError(f"不支持的优化器类型: {self.optimizer_type}")

        # 创建损失函数
        loss_name = self._config.get('loss', 'CrossEntropyLoss')
        if loss_name == 'CrossEntropyLoss':
            self._criterion = nn.CrossEntropyLoss()
        elif loss_name == 'MSELoss':
            self._criterion = nn.MSELoss()
        else:
            raise ValueError(f"不支持的损失函数: {loss_name}")

        # 创建数据加载器
        train_datasets = self._datasets.get("train", [])
        if train_datasets:
            self._train_loader = DataLoader(
                train_datasets[0],
                batch_size=self.batch_size,
                shuffle=True
            )

        test_datasets = self._datasets.get("test", [])
        if test_datasets:
            self._test_loader = DataLoader(
                test_datasets[0],
                batch_size=self.batch_size,
                shuffle=False
            )

        self.logger.info(f"Setup完成: train_samples={len(train_datasets[0]) if train_datasets else 0}")

    def is_head_layer(self, param_name: str) -> bool:
        """判断参数是否属于头部层"""
        for layer_name in self.head_layer_names:
            if layer_name in param_name:
                return True
        return False

    def get_base_parameters(self) -> Dict[str, torch.Tensor]:
        """获取base layers的参数（用于联邦聚合）"""
        base_params = {}
        for name, param in self.model.named_parameters():
            if not self.is_head_layer(name):
                base_params[name] = param.data.cpu().clone()
        return base_params

    def get_features(self, model, data):
        """获取模型的特征表示"""
        last_fc_candidates = ['fc2', 'fc', 'classifier', 'head']

        last_fc_name = None
        for name in last_fc_candidates:
            if hasattr(model, name):
                last_fc_name = name
                break

        if last_fc_name:
            original_fc = getattr(model, last_fc_name)
            setattr(model, last_fc_name, nn.Identity())
            features = model(data)
            setattr(model, last_fc_name, original_fc)
            return features
        else:
            # fallback
            features = []

            def hook_fn(module, input, output):
                features.append(output)

            modules = list(model.children())
            if len(modules) >= 2:
                handle = modules[-2].register_forward_hook(hook_fn)
                _ = model(data)
                handle.remove()

                if features:
                    feat = features[0]
                    if len(feat.shape) > 2:
                        feat = torch.flatten(feat, 1)
                    return feat

            output = model(data)
            if len(output.shape) > 2:
                output = torch.flatten(output, 1)
            return output

    def contrastive_loss(self, features, labels):
        """
        计算监督对比损失

        Args:
            features: 特征向量 (batch_size, feature_dim)
            labels: 真实标签 (batch_size,)

        Returns:
            对比损失值
        """
        # L2归一化
        features = F.normalize(features, dim=1)

        # 计算相似度矩阵
        similarity_matrix = torch.matmul(features, features.T) / self.temperature

        # 创建标签mask：同类为1，不同类为0
        batch_size = labels.shape[0]
        labels = labels.contiguous().view(-1, 1)
        mask = torch.eq(labels, labels.T).float().to(self.device)

        # 排除自己与自己的相似度
        logits_mask = torch.scatter(
            torch.ones_like(mask),
            1,
            torch.arange(batch_size).view(-1, 1).to(self.device),
            0
        )
        mask = mask * logits_mask

        # 计算对比损失
        exp_logits = torch.exp(similarity_matrix) * logits_mask
        log_prob = similarity_matrix - torch.log(exp_logits.sum(1, keepdim=True))

        # 计算平均损失（只对同类样本）
        mean_log_prob_pos = (mask * log_prob).sum(1) / (mask.sum(1) + 1e-6)
        loss = -mean_log_prob_pos.mean()

        return loss

    async def train_epoch(self, epoch_idx: int) -> EpochMetrics:
        """单轮训练 - FedCP训练循环"""
        self.model.train()

        total_loss = 0.0
        total_ce_loss = 0.0
        total_contrast_loss = 0.0
        total_correct = 0
        total_samples = 0

        for data, target in self._train_loader:
            data, target = data.to(self.device), target.to(self.device)

            self._optimizer.zero_grad()

            # 前向传播
            output = self.model(data)
            ce_loss = self._criterion(output, target)

            # 计算对比损失
            features = self.get_features(self.model, data)
            contrast_loss = self.contrastive_loss(features, target)

            # 总损失
            loss = ce_loss + self.lambda_contrast * contrast_loss

            loss.backward()
            self._optimizer.step()

            total_loss += loss.item() * data.size(0)
            total_ce_loss += ce_loss.item() * data.size(0)
            total_contrast_loss += contrast_loss.item() * data.size(0)

            pred = output.argmax(dim=1, keepdim=True)
            total_correct += pred.eq(target.view_as(pred)).sum().item()
            total_samples += data.size(0)

        avg_loss = total_loss / total_samples
        avg_accuracy = total_correct / total_samples

        self.logger.info(
            f"[{self._node_id}] Epoch {epoch_idx}: "
            f"Loss={avg_loss:.4f} (CE={total_ce_loss/total_samples:.4f}, "
            f"Contrast={total_contrast_loss/total_samples:.4f}), Acc={avg_accuracy:.4f}"
        )

        epoch_metrics = EpochMetrics(
            epoch=epoch_idx,
            avg_loss=avg_loss,
            total_samples=total_samples,
            metrics={
                'accuracy': avg_accuracy,
                'ce_loss': total_ce_loss / total_samples,
                'contrast_loss': total_contrast_loss / total_samples
            }
        )

        # 增加全局 epoch 计数器（用于 MLflow step）
        self._global_epoch_counter += 1

        # 触发 epoch 结束回调
        if self._callbacks:
            await self._callbacks.on_epoch_end(self, epoch_idx, epoch_metrics)

        return epoch_metrics

    async def evaluate(self, config: Optional[Dict] = None) -> EvalResult:
        """评估模型"""
        self.model.eval()

        loader = self._test_loader if self._test_loader else self._train_loader

        total_loss = 0.0
        total_correct = 0
        total_samples = 0

        with torch.no_grad():
            for data, target in loader:
                data, target = data.to(self.device), target.to(self.device)
                output = self.model(data)
                loss = self._criterion(output, target)

                total_loss += loss.item() * data.size(0)
                pred = output.argmax(dim=1, keepdim=True)
                total_correct += pred.eq(target.view_as(pred)).sum().item()
                total_samples += data.size(0)

        return EvalResult(
            num_samples=total_samples,
            metrics={
                'loss': total_loss / total_samples,
                'accuracy': total_correct / total_samples
            }
        )

    async def get_weights(self) -> Dict[str, Any]:
        """获取模型权重 - 只返回base parameters"""
        return self.get_base_parameters()

    async def set_weights(self, weights: Dict[str, Any]):
        """设置模型权重 - 只更新base parameters"""
        # 只更新base parameters，保留personalized head
        state_dict = self.model.state_dict()
        updated_count = 0

        for name, value in weights.items():
            if name in state_dict and not self.is_head_layer(name):
                if not isinstance(value, torch.Tensor):
                    value = torch.from_numpy(value)
                state_dict[name] = value.to(self.device)
                updated_count += 1

        self.model.load_state_dict(state_dict, strict=True)

        self.logger.debug(
            f"[{self._node_id}] FedCP: Updated {updated_count} base parameters, "
            f"kept personalized head unchanged"
        )


"""
传输层配置

包含 gRPC 和 Memory 传输的配置类及解析方法。

结构：
    TransportConfig
    ├── GrpcConfig
    │   └── TlsConfig
    └── MemoryTransportConfig
"""

from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from .defaults import (
    DEFAULT_TRAINER_PORT,
    DEFAULT_TIMEOUT,
    DEFAULT_CONNECTION_TIMEOUT,
    DEFAULT_MAX_RETRIES,
    DEFAULT_RETRY_INTERVAL,
    DEFAULT_BACKOFF_FACTOR,
    DEFAULT_HEARTBEAT_INTERVAL,
    DEFAULT_HEARTBEAT_TIMEOUT,
)


# ==================== TLS 配置 ====================

@dataclass
class TlsConfig:
    """TLS 配置"""
    enabled: bool = False
    cert_file: Optional[str] = None
    key_file: Optional[str] = None
    ca_file: Optional[str] = None
    mutual_tls: bool = False


def parse_tls_config(data: Optional[Dict[str, Any]]) -> TlsConfig:
    """解析 TLS 配置"""
    if not data:
        return TlsConfig()
    return TlsConfig(
        enabled=data.get("enabled", False),
        cert_file=data.get("cert_file"),
        key_file=data.get("key_file"),
        ca_file=data.get("ca_file"),
        mutual_tls=data.get("mutual_tls", False),
    )


# ==================== gRPC 配置 ====================

@dataclass
class GrpcConfig:
    """
    gRPC 配置
    
    Attributes:
        host: 绑定地址
        port: 端口
        max_message_size: 最大消息大小（字节）
        max_workers: 最大工作线程数
        tls: TLS 配置
        
        # 心跳配置（gRPC 层）
        heartbeat_enabled: 是否启用心跳
        heartbeat_interval: 心跳间隔（秒）
        heartbeat_timeout: 心跳超时（秒）
        heartbeat_check_interval: 健康检查间隔（秒）
        
        # 连接管理
        max_connection_wait_time: 最大连接等待时间（秒）
        auto_shutdown_on_failure: 连接失败后是否自动关闭
        critical_peers: 关键节点列表
        
        # 双线程优化
        dual_thread_enabled: 是否启用双线程模式
    """
    # 基础配置
    host: str = "0.0.0.0"
    port: int = DEFAULT_TRAINER_PORT
    max_message_size: int = 100 * 1024 * 1024  # 100MB
    max_workers: int = 10
    tls: TlsConfig = field(default_factory=TlsConfig)
    
    # 心跳配置
    heartbeat_enabled: bool = True
    heartbeat_interval: float = 5.0
    heartbeat_timeout: float = DEFAULT_HEARTBEAT_INTERVAL
    heartbeat_check_interval: float = DEFAULT_HEARTBEAT_TIMEOUT
    
    # 连接管理
    max_connection_wait_time: float = DEFAULT_TIMEOUT
    auto_shutdown_on_failure: bool = True
    critical_peers: List[str] = field(default_factory=list)
    
    # 双线程优化
    dual_thread_enabled: bool = True
    
    def __post_init__(self):
        """初始化后处理"""
        if isinstance(self.tls, dict):
            self.tls = TlsConfig(**self.tls)
    
    def get_address(self) -> str:
        """获取完整地址"""
        return f"{self.host}:{self.port}"
    
    # 向后兼容别名
    @property
    def max_message_length(self) -> int:
        """向后兼容属性"""
        return self.max_message_size


def parse_grpc_config(data: Optional[Dict[str, Any]]) -> GrpcConfig:
    """解析 gRPC 配置"""
    if not data:
        return GrpcConfig()
    
    # 解析 TLS 配置
    tls_config = parse_tls_config(data.get("tls"))
    
    # 从 heartbeat 子配置或直接字段获取心跳配置
    heartbeat = data.get("heartbeat", {})
    
    # 从 dual_thread 子配置获取双线程配置
    dual_thread = data.get("dual_thread", {})
    
    return GrpcConfig(
        host=data.get("host", "0.0.0.0"),
        port=data.get("port", DEFAULT_TRAINER_PORT),
        max_message_size=data.get("max_message_size", 100 * 1024 * 1024),
        max_workers=data.get("max_workers", 10),
        tls=tls_config,
        # 心跳配置
        heartbeat_enabled=heartbeat.get("enabled", True),
        heartbeat_interval=heartbeat.get("interval", 5.0),
        heartbeat_timeout=heartbeat.get("timeout", DEFAULT_HEARTBEAT_INTERVAL),
        heartbeat_check_interval=heartbeat.get("check_interval", DEFAULT_HEARTBEAT_TIMEOUT),
        # 连接管理
        max_connection_wait_time=heartbeat.get("max_connection_wait_time", DEFAULT_TIMEOUT),
        auto_shutdown_on_failure=heartbeat.get("auto_shutdown_on_failure", True),
        critical_peers=heartbeat.get("critical_peers", []),
        # 双线程优化
        dual_thread_enabled=dual_thread.get("enabled", True),
    )


# ==================== Memory 传输配置 ====================

@dataclass
class MemoryTransportConfig:
    """Memory 传输配置"""
    zero_copy: bool = True
    simulate_latency: bool = False
    latency_ms: float = 0.0


def parse_memory_config(data: Optional[Dict[str, Any]]) -> MemoryTransportConfig:
    """解析 Memory 传输配置"""
    if not data:
        return MemoryTransportConfig()
    return MemoryTransportConfig(
        zero_copy=data.get("zero_copy", True),
        simulate_latency=data.get("simulate_latency", False),
        latency_ms=data.get("latency_ms", 0.0),
    )


# ==================== 传输层配置 ====================

@dataclass
class TransportConfig:
    """
    传输层配置
    
    Attributes:
        mode: 传输模式（memory 或 grpc）
        grpc: gRPC 配置
        memory: Memory 传输配置
    """
    mode: str = "memory"
    grpc: GrpcConfig = field(default_factory=GrpcConfig)
    memory: MemoryTransportConfig = field(default_factory=MemoryTransportConfig)
    
    def __post_init__(self):
        """初始化后处理"""
        if isinstance(self.grpc, dict):
            self.grpc = GrpcConfig(**self.grpc)
        if isinstance(self.memory, dict):
            self.memory = MemoryTransportConfig(**self.memory)


def parse_transport_config(data: Optional[Dict[str, Any]]) -> TransportConfig:
    """解析传输层配置"""
    if not data:
        return TransportConfig()
    
    return TransportConfig(
        mode=data.get("mode", "memory"),
        grpc=parse_grpc_config(data.get("grpc")),
        memory=parse_memory_config(data.get("memory")),
    )


# ==================== 连接重试配置 ====================

@dataclass
class ConnectionRetryConfig:
    """
    连接重试配置
    
    Attributes:
        enabled: 是否启用重试
        max_retries: 最大重试次数（-1 表示无限重试）
        retry_interval: 重试间隔（秒）
        timeout: 总超时时间（秒）
        backoff: 退避策略
        backoff_factor: 退避因子
    """
    enabled: bool = True
    max_retries: int = DEFAULT_MAX_RETRIES
    retry_interval: float = DEFAULT_RETRY_INTERVAL
    timeout: float = DEFAULT_CONNECTION_TIMEOUT
    backoff: str = "exponential"
    backoff_factor: float = DEFAULT_BACKOFF_FACTOR
    
    def to_dict(self) -> Dict[str, Any]:
        """转换为字典（用于传递给 Node.connect）"""
        return {
            "enabled": self.enabled,
            "max_retries": self.max_retries,
            "retry_interval": self.retry_interval,
            "timeout": self.timeout,
            "backoff": self.backoff,
            "backoff_factor": self.backoff_factor,
        }


def parse_connection_retry_config(data: Optional[Dict[str, Any]]) -> ConnectionRetryConfig:
    """解析连接重试配置"""
    if not data:
        return ConnectionRetryConfig()
    return ConnectionRetryConfig(
        enabled=data.get("enabled", True),
        max_retries=data.get("max_retries", DEFAULT_MAX_RETRIES),
        retry_interval=data.get("retry_interval", DEFAULT_RETRY_INTERVAL),
        timeout=data.get("timeout", DEFAULT_CONNECTION_TIMEOUT),
        backoff=data.get("backoff", "exponential"),
        backoff_factor=data.get("backoff_factor", DEFAULT_BACKOFF_FACTOR),
    )


# ==================== 导出 ====================

__all__ = [
    # 配置类
    "TlsConfig",
    "GrpcConfig",
    "MemoryTransportConfig",
    "TransportConfig",
    "ConnectionRetryConfig",
    # 解析方法
    "parse_tls_config",
    "parse_grpc_config",
    "parse_memory_config",
    "parse_transport_config",
    "parse_connection_retry_config",
]



"""
Pickle 序列化器
"""

import pickle
import sys
from typing import Any

from .base import Serializer


class PickleSerializer(Serializer):
    """Pickle 序列化器"""

    @property
    def name(self) -> str:
        return "pickle"

    def serialize(self, obj: Any) -> bytes:
        """序列化对象为字节流"""
        return pickle.dumps(obj, protocol=pickle.HIGHEST_PROTOCOL)

    def deserialize(self, data: bytes) -> Any:
        """从字节流反序列化对象"""
        if not data:
            return None
        try:
            return pickle.loads(data)
        except ModuleNotFoundError as e:
            # 调试：分析 pickle 数据
            import pickletools
            import io
            msg = ''
            output = io.StringIO()
            pickletools.dis(data, output)
            # 只打印包含 federation 的行
            for line in output.getvalue().split('\n'):
                if 'federation' in line.lower():
                    msg += line + '\n'
            
            raise ModuleNotFoundError(
                f"反序列化失败，缺少模块: {e}. "
                f"Pickle 反序列化内容相关行:\n{msg}"
            ) from e


"""
MLflow Tracker

使用 MLflow 记录实验
"""

import sys
from typing import Dict, Any, Optional
from .base import Tracker
from ..registry import register
from ..infra import get_module_logger

logger = get_module_logger(__name__)


def _print_error(msg: str) -> None:
    """打印错误到 stderr，确保用户能看到"""
    print(f"[MLflowTracker] ❌ {msg}", file=sys.stderr)


def _print_warning(msg: str) -> None:
    """打印警告到 stderr"""
    print(f"[MLflowTracker] ⚠️  {msg}", file=sys.stderr)


def _print_info(msg: str) -> None:
    """打印信息"""
    print(f"[MLflowTracker] ✓ {msg}")


@register("federated.tracker.mlflow")
class MLflowTracker(Tracker):
    """
    MLflow Tracker

    使用 MLflow 记录实验
    """

    def __init__(
        self,
        tracking_uri: str = "./mlruns",
        experiment_name: str = "federated_learning",
        run_name: str = None,
        auto_end_run: bool = True,
        auto_start_run: bool = True,
        node_id: str = None,  # 新增：节点 ID，用于指标前缀
        username: str = None,  # MLflow 认证用户名
        password: str = None,  # MLflow 认证密码
    ):
        """
        Args:
            tracking_uri: MLflow 存储路径
            experiment_name: 实验名称
            run_name: 运行名称（可选）
            auto_end_run: 是否自动结束运行
            auto_start_run: 是否自动启动运行（Trainer=True，Learner=False）
            node_id: 节点 ID，用于给指标添加前缀（格式：{node_id}/metric_name）
            username: MLflow 认证用户名（可选）
            password: MLflow 认证密码（可选）
        """
        self.node_id = node_id
        self.mlflow = None
        self.run_id = None
        self.tracking_uri = tracking_uri
        self.experiment_name = experiment_name
        self.run_name = run_name
        self.auto_end_run = auto_end_run
        self.auto_start_run = auto_start_run
        self._init_error: Optional[str] = None  # 记录初始化错误
        
        try:
            import mlflow
            import os

            # 设置 MLflow 认证环境变量（如果提供）
            if username:
                os.environ['MLFLOW_TRACKING_USERNAME'] = username
            if password:
                os.environ['MLFLOW_TRACKING_PASSWORD'] = password

            self.mlflow = mlflow

            # 设置 MLflow
            mlflow.set_tracking_uri(tracking_uri)

            # 创建或获取实验（处理各种边界情况）
            from mlflow.tracking import MlflowClient
            client = MlflowClient(tracking_uri=tracking_uri)
            
            experiment = mlflow.get_experiment_by_name(experiment_name)
            if experiment is None:
                # 实验不存在，创建新实验
                mlflow.create_experiment(experiment_name)
            elif experiment.lifecycle_stage == "deleted":
                # 实验已被软删除，尝试恢复
                try:
                    _print_info(f"Restoring deleted experiment: {experiment_name}")
                    client.restore_experiment(experiment.experiment_id)
                except Exception as e:
                    # 恢复失败（可能已被永久删除），创建新实验
                    _print_warning(f"Failed to restore experiment: {e}")
                    _print_info(f"Creating new experiment: {experiment_name}")
                    try:
                        mlflow.create_experiment(experiment_name)
                    except Exception as create_err:
                        # 如果创建也失败，使用带时间戳的新名称
                        import time
                        new_name = f"{experiment_name}_{int(time.time())}"
                        _print_warning(f"Creating experiment with new name: {new_name}")
                        mlflow.create_experiment(new_name)
                        experiment_name = new_name
            
            mlflow.set_experiment(experiment_name)

            # Trainer 端：立即启动 run
            if auto_start_run:
                # 确保没有活动的 run（清理残留）
                if mlflow.active_run() is not None:
                    _print_warning("Ending previous active run before starting new one")
                    mlflow.end_run()

                mlflow.start_run(run_name=run_name)
                self.run_id = mlflow.active_run().info.run_id
                _print_info(
                    f"Initialized: run_id={self.run_id}, "
                    f"uri={tracking_uri}, experiment={experiment_name}"
                )
                logger.info(
                    f"MLflowTracker initialized with run_id: {self.run_id}, "
                    f"tracking_uri={tracking_uri}, experiment={experiment_name}"
                )
            else:
                _print_info(
                    f"Initialized (waiting for run_id): "
                    f"uri={tracking_uri}, experiment={experiment_name}"
                )
                logger.info(
                    f"MLflowTracker initialized (no active run): "
                    f"tracking_uri={tracking_uri}, experiment={experiment_name}"
                )

        except ImportError as e:
            error_msg = str(e)
            self._init_error = error_msg
            
            # 提供具体的解决建议
            if "databricks" in error_msg:
                _print_error(f"MLflow 依赖缺失: {error_msg}")
                _print_error("解决方法: pip install databricks-sdk")
            elif "mlflow" in error_msg.lower():
                _print_error(f"MLflow 未安装: {error_msg}")
                _print_error("解决方法: pip install mlflow")
            else:
                _print_error(f"导入错误: {error_msg}")
            
            _print_warning("MLflowTracker 已禁用，指标将不会记录到 MLflow")
            logger.warning(f"MLflowTracker disabled due to ImportError: {error_msg}")
            
        except Exception as e:
            error_msg = str(e)
            self._init_error = error_msg
            
            # 连接错误等
            _print_error(f"MLflow 初始化失败: {error_msg}")
            
            if "connection" in error_msg.lower() or "refused" in error_msg.lower():
                _print_error(f"无法连接到 MLflow server: {tracking_uri}")
                _print_error("请检查 MLflow server 是否运行，或使用本地路径如 './mlruns'")
            
            _print_warning("MLflowTracker 已禁用，指标将不会记录到 MLflow")
            logger.warning(f"MLflowTracker disabled due to error: {error_msg}")

    def log_metrics(self, metrics: Dict[str, float], step: int = None):
        """
        记录指标

        如果设置了 node_id，指标名称会自动添加前缀：{node_id}/metric_name
        例如：node_id="learner_0" 时，"accuracy" -> "learner_0/accuracy"

        注意：MLflow 只支持标量值（int/float），复杂类型（list/dict/tensor）会被跳过
        """
        if not self.mlflow:
            logger.debug("MLflow not available, skipping metric logging")
            return
            
        # 必须有 run_id（来自 Trainer 同步或自己创建）
        if not self.run_id:
            logger.warning("No run_id available. Skipping metric logging.")
            return

        # 展平和过滤指标
        flat_metrics = self._flatten_metrics(metrics)
        logger.debug(f"Logging {len(flat_metrics)} metrics to MLflow")

        for key, value in flat_metrics.items():
            try:
                # 如果有 node_id，添加前缀
                metric_name = f"{self.node_id}/{key}" if self.node_id else key

                # 使用 client API 直接指定 run_id 记录
                from mlflow.tracking import MlflowClient
                client = MlflowClient(tracking_uri=self.tracking_uri)
                client.log_metric(self.run_id, metric_name, value, step=step or 0)
                
            except Exception as e:
                logger.error(f"Failed to log metric {key}: {e}")

    def _flatten_metrics(self, metrics: Dict[str, Any], prefix: str = "") -> Dict[str, float]:
        """
        展平指标字典，并过滤出标量值

        Args:
            metrics: 原始指标字典（可能包含嵌套结构）
            prefix: 键前缀（用于递归）

        Returns:
            展平后的指标字典，只包含 int/float 值
        """
        flat = {}

        for key, value in metrics.items():
            # 构造完整的键名
            full_key = f"{prefix}/{key}" if prefix else key

            # 检查值的类型
            if isinstance(value, (int, float)) and not isinstance(value, bool):
                # 标量值：直接记录
                flat[full_key] = float(value)
            elif isinstance(value, dict):
                # 嵌套字典：递归展平
                nested = self._flatten_metrics(value, prefix=full_key)
                flat.update(nested)
            elif isinstance(value, (list, tuple)):
                # 列表/元组：跳过（MLflow 不支持）
                logger.debug(f"Skipping metric '{full_key}' (type: {type(value).__name__})")
            elif hasattr(value, 'item'):
                # NumPy/Torch 标量：提取值
                try:
                    flat[full_key] = float(value.item())
                except (AttributeError, ValueError):
                    logger.debug(f"Skipping metric '{full_key}' (cannot convert to float)")
            else:
                # 其他类型（如 tensor/array）：跳过
                logger.debug(f"Skipping metric '{full_key}' (type: {type(value).__name__})")

        return flat

    def log_params(self, params: Dict[str, Any]):
        """
        记录参数
        
        如果设置了 node_id，参数名称会自动添加前缀：{node_id}/param_name
        例如：node_id="learner_0" 时，"learning_rate" -> "learner_0/learning_rate"
        """
        print(f"[MLflowTracker] log_params: mlflow={self.mlflow is not None}, run_id={self.run_id}, node_id={self.node_id}")
        
        if not self.mlflow:
            print("[MLflowTracker] WARNING: mlflow not available")
            return
        
        # 必须有 run_id
        if not self.run_id:
            print("[MLflowTracker] WARNING: No run_id available. Skipping param logging.")
            return

        try:
            from mlflow.tracking import MlflowClient
            client = MlflowClient(tracking_uri=self.tracking_uri)
            
            print(f"[MLflowTracker] Logging {len(params)} params to run {self.run_id}")
            
            # MlflowClient.log_param 只能一个一个记录
            success_count = 0
            for key, value in params.items():
                try:
                    # 如果有 node_id，添加前缀（避免多 Learner 参数冲突）
                    param_name = f"{self.node_id}/{key}" if self.node_id else key
                    client.log_param(self.run_id, param_name, value)
                    success_count += 1
                except Exception as e:
                    # 参数可能已存在（MLflow 不允许覆盖）
                    print(f"[MLflowTracker] Failed to log param {key}: {e}")
            
            print(f"[MLflowTracker] Successfully logged {success_count}/{len(params)} params")
                    
        except Exception as e:
            print(f"[MLflowTracker] ERROR: Failed to log params: {e}")
            import traceback
            traceback.print_exc()

    def log_artifact(self, local_path: str, artifact_path: str = None):
        """记录文件"""
        if not self.mlflow:
            return
        
        # 必须有 run_id
        if not self.run_id:
            logger.debug("No run_id available. Skipping artifact logging.")
            return
        
        try:
            from mlflow.tracking import MlflowClient
            client = MlflowClient(tracking_uri=self.tracking_uri)
            client.log_artifact(self.run_id, local_path, artifact_path)
        except Exception as e:
            logger.error(f"Failed to log artifact {local_path}: {e}")

    def set_tags(self, tags: Dict[str, str]):
        """设置标签"""
        if not self.mlflow:
            return
        
        # 必须有 run_id
        if not self.run_id:
            logger.debug("No run_id available. Skipping tag setting.")
            return

        try:
            from mlflow.tracking import MlflowClient
            client = MlflowClient(tracking_uri=self.tracking_uri)
            
            for key, value in tags.items():
                try:
                    client.set_tag(self.run_id, key, value)
                except Exception as e:
                    logger.debug(f"Failed to set tag {key}: {e}")
                    
        except Exception as e:
            logger.error(f"Failed to set tags: {e}")

    def close(self):
        """结束运行"""
        if self.mlflow and self.auto_end_run:
            try:
                self.mlflow.end_run()
                _print_info("MLflow run ended")
                logger.info("MLflow run ended")
            except Exception as e:
                logger.error(f"Failed to end MLflow run: {e}")

    # ========== 状态检查 ==========

    @property
    def is_available(self) -> bool:
        """检查 MLflow 是否可用"""
        return self.mlflow is not None

    @property
    def init_error(self) -> Optional[str]:
        """获取初始化错误信息"""
        return self._init_error

    # ========== 分布式同步接口 ==========

    def get_sync_info(self) -> Dict[str, Any]:
        """
        获取同步信息（Trainer 端）

        Returns:
            包含 run_id 的字典，用于 Learner 同步
        """
        if self.run_id:
            return {"mlflow_run_id": self.run_id}
        return {}

    def join_run(self, run_id: str):
        """
        加入已有的 run（Learner 端）

        直接保存 run_id，之后使用 MlflowClient 记录指标，无需 start_run

        Args:
            run_id: Trainer 创建的 run_id
        """
        if not self.mlflow:
            _print_warning(f"Cannot join run {run_id}: MLflow not available")
            return

        # 如果已经有 run，先结束它（切换到 Trainer 的 run）
        if self.run_id is not None:
            if self.run_id == run_id:
                logger.info(f"Already in the same run {run_id}, no need to join")
                return
            
            # 结束当前 run（如果是活动的）
            try:
                if self.mlflow.active_run() is not None:
                    logger.info(f"Ending previous run {self.run_id} to join Trainer's run")
                    self.mlflow.end_run()
            except Exception as e:
                logger.warning(f"Failed to end previous run: {e}")

        # 保存新的 run_id，使用 MlflowClient 记录指标
        self.run_id = run_id
        _print_info(f"Joined MLflow run: {run_id}")
        logger.info(f"Joined MLflow run: {run_id}")


"""
联邦学习数据管理
"""

from .provider import DataProvider, InMemoryDataProvider, SubsetDataProvider
from .partitioner import (
    Partitioner,
    IIDPartitioner,
    LabelPartitioner,
    DirichletPartitioner,
    QuantityPartitioner,
)
from .manager import DataManager
from .partitioned_dataset import (
    create_partitioned_dataset,
    create_dataset_with_partition,
)

__all__ = [
    # 数据提供者
    "DataProvider",
    "InMemoryDataProvider",
    "SubsetDataProvider",

    # 划分器
    "Partitioner",
    "IIDPartitioner",
    "LabelPartitioner",
    "DirichletPartitioner",
    "QuantityPartitioner",

    # 数据管理器
    "DataManager",

    # 划分工厂函数
    "create_partitioned_dataset",
    "create_dataset_with_partition",
]



"""
Node 通信节点核心实现
"""

import asyncio
import time
import traceback
import uuid
from collections import defaultdict
from dataclasses import dataclass, field
from typing import Any, Awaitable, Callable, Dict, List, Optional, Set, Union

# 导入配置类（从统一的 config 模块）
from ..config import MethodOptions
# 从主配置模块导入 NodeCommConfig（统一的配置定义）
from ..config import NodeCommConfig
from .message import (
    Message,
    MessageType,
    MessageContext,
    ConnectionInfo,
    ConnectionStatus,
    ErrorInfo,
)
from .exceptions import (
    NodeError,
    NodeNotConnectedError,
    NodeDisconnectedError,
    CallTimeoutError,
    RemoteExecutionError,
    SerializationError,
    InterceptorAbort,
)
from .transport import Transport, create_transport
from .serialization import SerializerRegistry, Serializer
from .interceptor import InterceptorChain, LoggingInterceptor, AuthInterceptor, Interceptor

# 使用 loguru 统一日志（会继承调用方的 node_id 和 log_type）
from ..infra.logging import get_logger

# 类型定义
MethodHandler = Callable[[Any, MessageContext], Awaitable[Any]]
EventHandler = Callable[..., Any]


@dataclass
class MethodEntry:
    """方法注册条目"""
    handler: MethodHandler
    options: MethodOptions = field(default_factory=MethodOptions)


@dataclass
class PendingRequest:
    """等待响应的请求"""
    future: asyncio.Future
    method: str
    created_at: float
    timeout_at: float
    target: str


class Node:
    """
    通信节点
    
    核心职责：
    1. 方法注册与路由
    2. 消息发送与接收
    3. 连接管理
    4. 请求-响应匹配
    
    Example:
        async def handle_train(payload, ctx):
            return {"status": "completed"}
        
        node = Node("my_node", config)
        node.register("train", handle_train)
        await node.start()
    """
    
    def __init__(
        self,
        node_id_or_config: Union[str, NodeCommConfig],
        config: Optional[NodeCommConfig] = None,
    ):
        """
        创建 Node 实例
        
        支持两种调用方式:
        1. Node(node_id, config)  - 传统方式
        2. Node(config)           - 新方式，从 config.node_id 获取 ID
        
        Args:
            node_id_or_config: 节点 ID（字符串）或 NodeCommConfig 实例
            config: 配置对象（仅当第一个参数是 node_id 时使用）
        """
        # 处理两种调用方式
        if isinstance(node_id_or_config, str):
            # 传统方式: Node(node_id, config)
            self.node_id = node_id_or_config
            self.config = config or NodeCommConfig(node_id=self.node_id)
        else:
            # 新方式: Node(config)
            self.config = node_id_or_config
            self.node_id = self.config.node_id
        
        # 确保 config.node_id 一致
        if self.config.node_id != self.node_id:
            self.config.node_id = self.node_id
        
        # 创建绑定了 node_id 和 log_type 的 logger
        self.logger = get_logger(node_id=self.node_id, log_type="system")
        
        # 核心表
        self._method_table: Dict[str, MethodEntry] = {}
        self._pending_table: Dict[str, PendingRequest] = {}
        self._connection_table: Dict[str, ConnectionInfo] = {}

        # 服务对象表（用于 bind）
        self._services: Dict[str, Any] = {}

        # 事件处理器
        self._event_handlers: Dict[str, List[EventHandler]] = defaultdict(list)
        
        # 组件（延迟初始化）
        self._transport: Optional[Transport] = None
        self._serializer_registry: Optional[SerializerRegistry] = None
        self._interceptor_chain: Optional[InterceptorChain] = None
        
        # 状态
        self._running: bool = False
        self._accepting_requests: bool = True  # 是否接受新请求

        # In-flight请求跟踪
        self._in_flight_requests: int = 0
        self._in_flight_lock: asyncio.Lock = asyncio.Lock()

        # 后台任务
        self._background_tasks: List[asyncio.Task] = []
    
    # ==================== 事件 API ====================
    
    def on(self, event: str, handler: EventHandler) -> "Node":
        """
        注册事件处理器，支持链式调用
        
        事件:
            "connect"    - handler(node_id: str, address: Optional[str])
            "disconnect" - handler(node_id: str, reason: str)
        
        Example:
            node.on("connect", lambda nid, addr: print(f"Connected: {nid}"))
        """
        self._event_handlers[event].append(handler)
        return self
    
    def off(self, event: str, handler: Optional[EventHandler] = None) -> "Node":
        """移除事件处理器，不传 handler 则移除该事件所有处理器"""
        if handler is None:
            self._event_handlers[event].clear()
        else:
            try:
                self._event_handlers[event].remove(handler)
            except ValueError:
                pass
        return self
    
    async def _emit(self, event: str, *args, **kwargs) -> None:
        """
        触发事件

        统一处理同步和异步事件处理器：
        - 异步处理器：直接 await
        - 同步处理器：在线程池中执行，避免阻塞事件循环
        """
        for handler in self._event_handlers[event]:
            try:
                # 判断是否为协程函数
                if asyncio.iscoroutinefunction(handler):
                    # 异步处理器：直接 await
                    await handler(*args, **kwargs)
                else:
                    # 同步处理器：在线程池中执行
                    await asyncio.to_thread(handler, *args, **kwargs)
            except Exception as e:
                self.logger.error(f"Event handler error [{event}]: {e}")
    
    # ==================== 方法注册 ====================
    
    def register(
        self,
        method: str,
        handler: MethodHandler,
        options: Optional[MethodOptions] = None,
    ) -> None:
        """
        注册方法处理器

        Args:
            method: 方法名
            handler: 处理函数，签名为 async def handler(payload, context) -> result
            options: 方法选项

        Example:
            async def handle_train(payload, ctx):
                epochs = payload["epochs"]
                return {"status": "completed", "loss": 0.01}

            node.register("train", handle_train)
        """
        if method in self._method_table:
            raise ValueError(f"Method '{method}' already registered")

        self._method_table[method] = MethodEntry(
            handler=handler,
            options=options or MethodOptions(),
        )
        self.logger.debug(f"Registered method: {method}")
    
    def unregister(self, method: str) -> None:
        """注销方法"""
        self._method_table.pop(method, None)
        self.logger.debug(f"Unregistered method: {method}")
    
    def has_method(self, method: str) -> bool:
        """检查方法是否已注册"""
        return method in self._method_table

    def bind(self, service: Any, name: str = "default") -> None:
        """
        暴露本地服务对象

        将服务对象的所有公开方法自动注册为可调用的远程方法。
        远程节点可以通过 Proxy 调用 service 的方法。

        Args:
            service: 要暴露的对象（如 Learner 实例）
            name: 服务名称（用于区分多个服务）

        Example:
            class Learner:
                async def fit(self, config):
                    return {"loss": 0.1}

                async def evaluate(self, config):
                    return {"accuracy": 0.9}

            learner = Learner()
            node.bind(learner, name="learner")

            # 远程可以调用: node.call(target, "fit", {...})
        """
        if name in self._services:
            raise ValueError(f"Service '{name}' already bound")

        self._services[name] = service

        # 收集要注册的方法名列表
        methods_to_register = []

        # 自动注册服务对象的所有公开方法
        for method_name in dir(service):
            # 跳过私有方法和魔术方法
            if method_name.startswith("_"):
                continue

            method = getattr(service, method_name)

            # 只注册可调用的方法
            if not callable(method):
                continue

            # 检查是否是协程函数
            is_async = asyncio.iscoroutinefunction(method)

            # 跳过既不是协程也不是普通函数的（如属性等）
            if not is_async and not callable(method):
                continue

            methods_to_register.append(method_name)

            # 创建处理器闭包
            if is_async:
                # 异步方法：直接调用
                async def handler(payload: Any, ctx: MessageContext, m=method) -> Any:
                    """调用服务对象的异步方法"""
                    if isinstance(payload, dict):
                        # 检查是否是标准格式 {"args": (), "kwargs": {}}
                        if "args" in payload or "kwargs" in payload:
                            args = payload.get("args", ())
                            kwargs = payload.get("kwargs", {})
                            return await m(*args, **kwargs)
                        else:
                            # 普通字典，作为 **kwargs 传递
                            return await m(**payload)
                    else:
                        # 单一参数
                        return await m(payload)
            else:
                # 同步方法：包装成协程
                async def handler(payload: Any, ctx: MessageContext, m=method) -> Any:
                    """调用服务对象的同步方法（包装为协程）"""
                    if isinstance(payload, dict):
                        # 检查是否是标准格式 {"args": (), "kwargs": {}}
                        if "args" in payload or "kwargs" in payload:
                            args = payload.get("args", ())
                            kwargs = payload.get("kwargs", {})
                            # 在线程池中执行同步方法
                            return await asyncio.to_thread(m, *args, **kwargs)
                        else:
                            # 普通字典，作为 **kwargs 传递
                            return await asyncio.to_thread(m, **payload)
                    else:
                        # 单一参数
                        return await asyncio.to_thread(m, payload)

            # 注册方法（Memory 模式默认使用 pickle 序列化器）
            # 检查是否是 Memory 模式
            is_memory_mode = (
                hasattr(self, 'config') and
                hasattr(self.config, 'transport') and
                self.config.transport.mode == 'memory'
            )
            options = MethodOptions(serializer='pickle') if is_memory_mode else None
            self.register(method_name, handler, options=options)
            if is_memory_mode:
                self.logger.debug(f"Registered method '{method_name}' with pickle serializer (Memory mode)")

        self.logger.debug(f"服务 '{name}' 已绑定，暴露方法: {methods_to_register}")

    def unbind(self, name: str = "default") -> None:
        """
        解除服务绑定

        Args:
            name: 服务名称
        """
        service = self._services.pop(name, None)
        if service:
            # 注销所有相关方法
            for method_name in dir(service):
                if not method_name.startswith("_") and callable(getattr(service, method_name)):
                    self.unregister(method_name)
            self.logger.debug(f"Service '{name}' unbound")
    
    # ==================== 消息发送 ====================
    
    async def call(
        self,
        target: str,
        method: str,
        payload: Any,
        timeout: Optional[float] = None,
        metadata: Optional[Dict[str, str]] = None,
        serializer: Optional[str] = None,
        use_smart_wait: bool = True,  # 新增：是否使用智能等待（基于状态）
    ) -> Any:
        """
        调用远程方法（同步等待响应）

        Args:
            target: 目标节点 ID
            method: 方法名
            payload: 请求参数
            timeout: 超时时间（秒），None 使用 MethodOptions 或默认值
            metadata: 附加元数据
            serializer: 序列化器名称（如 "json", "pickle"），None 使用默认值
            use_smart_wait: 是否使用智能等待（基于节点状态），默认True

        Timeout 优先级:
            显式参数 > MethodOptions.timeout > NodeConfig.default_timeout

        Returns:
            远程方法的返回值

        Raises:
            CallTimeoutError: 调用超时
            NodeNotConnectedError: 目标节点未连接
            RemoteExecutionError: 远程执行失败

        Example:
            result = await node.call(
                target="participant_1",
                method="train",
                payload={" epochs": 10, "lr": 0.001},
                timeout=300.0,
            )
        """
        # DEBUG: 记录调用开始
        self.logger.debug(f"[RPC-SEND] {target}.{method}(), payload={payload.keys()}")

        # 1. 检查连接
        if not self._is_connected(target):
            raise NodeNotConnectedError(target)

        # 2. 确定超时（优先级：显式 > MethodOptions > NodeConfig）
        if timeout is None:
            entry = self._method_table.get(method)
            if entry and entry.options.timeout is not None:
                timeout = entry.options.timeout
            else:
                timeout = self.config.default_timeout

        # 3. 序列化 payload
        ser = self._get_serializer(method, serializer)
        try:
            payload_bytes = ser.serialize(payload)
        except Exception as e:
            raise SerializationError(f"Failed to serialize payload: {e}")

        # 4. 构造消息
        message_id = str(uuid.uuid4())
        message = Message(
            id=message_id,
            type=MessageType.REQUEST,
            method=method,
            source=self.node_id,
            target=target,
            payload=payload_bytes,
            correlation_id=None,
            timestamp=time.time_ns(),
            metadata=metadata or {},
            error=None,
        )

        self.logger.debug(f"构造消息: id={message_id}, target={target}, method={method}")

        # 5. 创建 Future 并注册到 pending_table
        loop = asyncio.get_event_loop()
        future: asyncio.Future = loop.create_future()
        now = time.time()
        pending = PendingRequest(
            future=future,
            method=method,
            created_at=now,
            timeout_at=now + timeout,
            target=target,
        )
        self._pending_table[message_id] = pending

        try:
            # 6. 通过 Interceptor 链处理
            message = await self._interceptor_chain.execute_outbound(message, self.node_id)

            # 7. 发送消息
            self.logger.debug(f"发送消息: id={message_id}, target={target}, method={method}")
            await self._transport.send(message)
            self.logger.debug(f"消息已发送: id={message_id}")

            # 8. 等待响应（智能等待或固定超时）
            if use_smart_wait and method not in ["_fed_get_property", "_heartbeat"]:
                # 使用智能等待（基于状态）
                response: Message = await self._smart_wait_for_response(
                    future=future,
                    target=target,
                    method=method,
                    message_id=message_id,
                    initial_timeout=timeout
                )
            else:
                # 使用固定超时（用于内部方法和快速调用）
                response: Message = await asyncio.wait_for(future, timeout=timeout)

            self.logger.debug(f"收到响应: id={message_id}, has_error={response.error is not None}")

            # 9. 检查错误
            if response.error:
                raise RemoteExecutionError(
                    code=response.error.code,
                    message=response.error.message,
                    details=response.error.details,
                )

            # 10. 反序列化结果
            result = ser.deserialize(response.payload)
            self.logger.debug(f"[RPC-RECV] ← {target}.{method}() = {type(result).__name__}")
            return result

        except asyncio.TimeoutError:
            self.logger.error(f"调用超时: target={target}, method={method}, timeout={timeout}")
            raise CallTimeoutError(
                msg_id=message_id,
                method=method,
                target=target,
                timeout=timeout,
            )
        finally:
            # 11. 清理 pending_table
            self._pending_table.pop(message_id, None)

    async def _smart_wait_for_response(
        self,
        future: asyncio.Future,
        target: str,
        method: str,
        message_id: str,
        initial_timeout: float
    ) -> Message:
        """
        智能等待响应：基于远程节点状态而非固定超时

        策略：
        1. 周期性检查future是否完成（响应已到达）
        2. 如果未完成，查询目标节点的状态
        3. 如果节点处于工作状态（TRAINING/EVALUATING/AGGREGATING），继续等待
        4. 如果节点处于IDLE状态但响应未到，可能有问题，继续等待但记录警告
        5. 如果节点UNHEALTHY，抛出超时
        6. 使用initial_timeout作为基准，但在节点工作时可以延长

        Args:
            future: 等待的Future对象
            target: 目标节点ID
            method: 调用的方法名
            message_id: 消息ID
            initial_timeout: 初始超时时间（秒）

        Returns:
            Message: 响应消息

        Raises:
            asyncio.TimeoutError: 真正的超时（节点异常或无响应）
        """
        check_interval = 5.0  # 每5秒检查一次状态
        start_time = time.time()
        last_status_check = start_time
        last_logged_state = None
        max_idle_wait = 30.0  # 节点IDLE状态下最多额外等待30秒
        idle_start_time = None

        while True:
            try:
                # 尝试等待响应（带短超时）
                response = await asyncio.wait_for(future, timeout=check_interval)
                return response

            except asyncio.TimeoutError:
                # 检查是否需要查询状态
                now = time.time()
                elapsed = now - start_time

                # 查询远程节点状态
                try:
                    # 调用远程的_fed_get_property获取state属性（使用固定超时，避免递归）
                    state_result = await asyncio.wait_for(
                        self.call(
                            target,
                            "_fed_get_property",
                            {"property_name": "state"},  # 注意：是"state"而不是"_state"
                            use_smart_wait=False  # 关键：避免递归
                        ),
                        timeout=3.0
                    )
                    remote_state = state_result if isinstance(state_result, str) else "unknown"

                except Exception as e:
                    # 无法获取状态，可能节点已崩溃
                    self.logger.warning(
                        f"[{self.node_id}] 无法获取 {target} 的状态: {e}, "
                        f"已等待 {elapsed:.1f}s"
                    )
                    remote_state = "unknown"

                # 根据状态决定是否继续等待
                if remote_state in ["training", "evaluating", "aggregating"]:
                    # 节点正在工作，继续等待
                    if remote_state != last_logged_state:
                        self.logger.info(
                            f"[{self.node_id}] {target}.{method}() 仍在执行中 "
                            f"(状态: {remote_state}, 已等待 {elapsed:.1f}s), 继续等待..."
                        )
                        last_logged_state = remote_state
                    idle_start_time = None  # 重置idle计时
                    continue

                elif remote_state == "idle":
                    # 节点已经IDLE，但响应还没到，可能在网络传输中
                    if idle_start_time is None:
                        idle_start_time = now
                        self.logger.debug(
                            f"[{self.node_id}] {target} 已进入IDLE状态，"
                            f"等待响应到达..."
                        )

                    idle_elapsed = now - idle_start_time
                    if idle_elapsed > max_idle_wait:
                        # IDLE状态等待太久，认为出问题了
                        self.logger.error(
                            f"[{self.node_id}] {target}.{method}() 调用超时: "
                            f"节点已IDLE {idle_elapsed:.1f}s 但响应未到达"
                        )
                        raise asyncio.TimeoutError()
                    continue

                elif remote_state == "unhealthy":
                    # 节点不健康，停止等待
                    self.logger.error(
                        f"[{self.node_id}] {target} 状态不健康，停止等待"
                    )
                    raise asyncio.TimeoutError()

                else:
                    # unknown或其他状态，使用初始超时作为兜底
                    if elapsed > initial_timeout * 2:  # 允许2倍的初始超时
                        self.logger.error(
                            f"[{self.node_id}] {target}.{method}() 调用超时: "
                            f"已等待 {elapsed:.1f}s (状态: {remote_state})"
                        )
                        raise asyncio.TimeoutError()
                    continue

    async def notify(
        self,
        target: str,
        method: str,
        payload: Any,
        metadata: Optional[Dict[str, str]] = None,
    ) -> None:
        """
        发送通知（不等待响应）
        
        Args:
            target: 目标节点 ID
            method: 方法名
            payload: 通知内容
            metadata: 附加元数据
            
        Example:
            await node.notify(
                target="coordinator",
                method="_heartbeat",
                payload={"status": "alive"},
            )
        """
        if not self._is_connected(target):
            raise NodeNotConnectedError(target)
        
        serializer = self._get_serializer(method)
        try:
            payload_bytes = serializer.serialize(payload)
        except Exception as e:
            raise SerializationError(f"Failed to serialize payload: {e}")
        
        message = Message(
            id=str(uuid.uuid4()),
            type=MessageType.NOTIFY,
            method=method,
            source=self.node_id,
            target=target,
            payload=payload_bytes,
            correlation_id=None,
            timestamp=time.time_ns(),
            metadata=metadata or {},
            error=None,
        )
        
        message = await self._interceptor_chain.execute_outbound(message, self.node_id)
        await self._transport.send(message)
    
    async def broadcast(
        self,
        targets: List[str],
        method: str,
        payload: Any,
        timeout: Optional[float] = None,
        wait_for_all: bool = True,
        min_responses: Optional[int] = None,
    ) -> Dict[str, Any]:
        """
        广播调用多个对等节点

        根据传输模式自动选择实现：
        - Memory 模式：串行调用（转换为多个单独调用）
        - gRPC 模式：并行调用（真正的网络广播）

        Args:
            targets: 目标节点 ID 列表
            method: 方法名
            payload: 参数（建议使用字典格式 {"args": (), "kwargs": {}}）
            timeout: 超时时间（秒）
            wait_for_all: 是否等待所有节点返回，False 时结合 min_responses 使用
            min_responses: 最少需要的响应数，达到后即可返回（仅在 wait_for_all=False 时生效）

        Returns:
            结果字典 {node_id: result}
            如果某个节点调用失败，对应的 value 为 Exception 对象

        Example:
            # 广播到所有连接的节点
            results = await node.broadcast(
                targets=["learner_1", "learner_2"],
                method="fit",
                payload={"args": (), "kwargs": {"epochs": 5}},
            )

            # 灵活广播：等待至少 3 个节点响应，超时 30 秒
            results = await node.broadcast(
                targets=all_learners,
                method="fit",
                payload={"args": (), "kwargs": {"epochs": 5}},
                timeout=30,
                wait_for_all=False,
                min_responses=3,
            )

            # 处理结果
            for node_id, result in results.items():
                if isinstance(result, Exception):
                    print(f"{node_id} failed: {result}")
                else:
                    print(f"{node_id} succeeded: {result}")
        """
        # 委托给 Transport 层处理
        return await self._transport.broadcast(
            targets, method, payload, timeout, wait_for_all, min_responses
        )

    
    # ==================== 消息接收与路由 ====================
    
    async def on_message(self, message: Message) -> Optional[Message]:
        """
        消息入口（Transport 调用此方法）

        Args:
            message: 接收到的消息

        Returns:
            对于 REQUEST，返回 RESPONSE
            对于 NOTIFY/RESPONSE，返回 None
        """
        # DEBUG: 记录消息到达节点
        self.logger.debug(f"on_message: type={message.type.value}, from={message.source}, method={message.method}, msg_id={message.id}")

        # 通过 Interceptor 链处理入站消息
        try:
            message = await self._interceptor_chain.execute_inbound(message, self.node_id)
        except InterceptorAbort as e:
            self.logger.warning(f"loggerInterceptor 中止消息: method={message.method}, error={e.error_info}")
            if message.type == MessageType.REQUEST:
                return self._create_error_response(message, e.error_info)
            return None

        # 根据消息类型分发
        if message.type == MessageType.REQUEST:
            self.logger.debug(f"logger分发到 _handle_request: method={message.method}")
            return await self._handle_request(message)
        elif message.type == MessageType.RESPONSE:
            self.logger.debug(f"logger分发到 _handle_response: correlation_id={message.correlation_id}")
            await self._handle_response(message)
            return None
        elif message.type == MessageType.NOTIFY:
            self.logger.debug(f"logger分发到 _handle_notify: method={message.method}")
            await self._handle_notify(message)
            return None

        return None
    
    async def _handle_request(self, message: Message) -> Message:
        """处理 REQUEST 消息（带 in-flight 跟踪）"""

        # DEBUG: 记录接收到请求
        self.logger.debug(f"[RPC-HANDLE] ← {message.source}.{message.method}() msg_id={message.id}")

        # 检查是否接受新请求
        if not self._accepting_requests:
            self.logger.warning(f"拒绝请求（节点关闭中）: method={message.method}")
            return self._create_error_response(
                message,
                ErrorInfo(
                    code="NODE_STOPPING",
                    message="Node is shutting down, not accepting new requests",
                ),
            )

        # 增加 in-flight 计数
        async with self._in_flight_lock:
            self._in_flight_requests += 1

        try:
            # 1. 查找 handler
            entry = self._method_table.get(message.method)
            if not entry:
                self.logger.error(f"方法未注册: method={message.method}")
                return self._create_error_response(
                    message,
                    ErrorInfo(
                        code="METHOD_NOT_FOUND",
                        message=f"Method '{message.method}' not registered",
                    ),
                )

            self.logger.debug(f"找到方法处理器: method={message.method}")

            # 2. 反序列化 payload
            try:
                serializer = self._get_serializer(message.method)
                payload = serializer.deserialize(message.payload)
                self.logger.debug(f"payload 反序列化成功: method={message.method}, payload_type={type(payload).__name__}")
            except Exception as e:
                self.logger.exception(f"Deserialization error for method {message.method}")
                return self._create_error_response(
                    message,
                    ErrorInfo(
                        code="SERIALIZATION_ERROR",
                        message=str(e),
                    ),
                )

            # 3. 构造上下文
            context = MessageContext(
                message=message,
                source=message.source,
                metadata=message.metadata,
            )

            # DEBUG: 打印payload的格式
            if message.method == "set_weights":
                self.logger.debug(f" set_weights payload type: {type(payload)}")
                if isinstance(payload, dict):
                    self.logger.debug(f" set_weights payload keys: {payload.keys()}")
                    if "args" in payload:
                        self.logger.debug(f" set_weights args: type={type(payload['args'])}, len={len(payload.get('args', ()))}")
                        if payload.get('args'):
                            self.logger.debug(f" set_weights first arg type: {type(payload['args'][0])}")
                    if "kwargs" in payload:
                        self.logger.debug(f" set_weights kwargs keys: {payload.get('kwargs', {}).keys()}")

            # 4. 执行 handler
            try:
                self.logger.debug(f"[RPC-EXEC] 执行 {message.method}()")
                result = await entry.handler(payload, context)
                self.logger.debug(f"[RPC-DONE] {message.method}() → {type(result).__name__}")
            except Exception as e:
                self.logger.error(f"handler执行失败: method={message.method}, error={e}")
                error_info = ErrorInfo(
                    code="EXECUTION_ERROR",
                    message=str(e),
                    stack_trace=traceback.format_exc() if self.config.debug else None,
                )
                return self._create_error_response(message, error_info)

            # 5. 构造响应
            try:
                result_bytes = serializer.serialize(result)
                # 调试：检查序列化后的字节
                if len(result_bytes) == 0:
                    self.logger.error(f"Serialized result is empty! method={message.method}, result={result}, result_type={type(result)}")
                self.logger.debug(f"结果序列化成功: method={message.method}, size={len(result_bytes)} bytes")
            except Exception as e:
                self.logger.exception(f"Serialization error for method {message.method}, result_type={type(result)}")
                return self._create_error_response(
                    message,
                    ErrorInfo(code="SERIALIZATION_ERROR", message=str(e)),
                )
        finally:
            # 减少 in-flight 计数
            async with self._in_flight_lock:
                self._in_flight_requests -= 1

        response = Message(
            id=str(uuid.uuid4()),
            type=MessageType.RESPONSE,
            method=message.method,
            source=self.node_id,
            target=message.source,
            payload=result_bytes,
            correlation_id=message.id,
            timestamp=time.time_ns(),
            metadata={},
            error=None,
        )

        self.logger.debug(f"发送响应: to={message.source}, method={message.method}, correlation_id={message.id}")
        return response
    
    async def _handle_response(self, message: Message) -> None:
        """处理 RESPONSE 消息"""

        # DEBUG: 记录响应处理
        self.logger.debug(f"_handle_response: from={message.source}, correlation_id={message.correlation_id}")

        pending = self._pending_table.get(message.correlation_id)
        if not pending:
            self.logger.warning(
                f"Orphan response received: correlation_id={message.correlation_id}"
            )
            return

        # 完成 Future
        if not pending.future.done():
            self.logger.debug(f"设置 Future 结果: correlation_id={message.correlation_id}, method={pending.method}")
            pending.future.set_result(message)
        else:
            self.logger.warning(f"Future 已完成: correlation_id={message.correlation_id}")
    
    async def _handle_notify(self, message: Message) -> None:
        """处理 NOTIFY 消息"""
        
        entry = self._method_table.get(message.method)
        if not entry:
            self.logger.warning(f"No handler for notify: method={message.method}")
            return
        
        try:
            serializer = self._get_serializer(message.method)
            payload = serializer.deserialize(message.payload)
            
            context = MessageContext(
                message=message,
                source=message.source,
                metadata=message.metadata,
            )
            
            await entry.handler(payload, context)
            
        except Exception as e:
            self.logger.error(f"Error handling notify {message.method}: {e}")
    
    def _create_error_response(
        self,
        request: Message,
        error: ErrorInfo,
    ) -> Message:
        """创建错误响应"""
        return Message(
            id=str(uuid.uuid4()),
            type=MessageType.RESPONSE,
            method=request.method,
            source=self.node_id,
            target=request.source,
            payload=b"",
            correlation_id=request.id,
            timestamp=time.time_ns(),
            metadata={},
            error=error,
        )
    
    # ==================== 连接管理 ====================
    
    async def connect(
        self,
        target: str,
        address: Optional[str] = None,
        retry_config: Optional[Dict[str, Any]] = None,
    ) -> None:
        """
        连接到目标节点（支持重试）

        Args:
            target: 目标节点 ID
            address: 目标地址（gRPC 模式需要）
            retry_config: 重试配置字典

        Example:
            # Memory 模式
            await node.connect("other_node")

            # gRPC 模式
            await node.connect("other_node", "192.168.1.100:50051")

            # 带重试配置
            await node.connect("other_node", retry_config={
                "enabled": True,
                "max_retries": 10,
                "retry_interval": 2.0,
            })
        """
        await self._transport.connect(target, address, retry_config)
        
        # 发送握手通知
        # 优先使用 advertised_address，否则从 listen 或 transport 配置构造
        handshake_address = self.config.advertised_address
        if not handshake_address:
            # 尝试从 listen 配置获取
            if hasattr(self.config, 'listen') and self.config.listen:
                host = self.config.listen.get("host", "localhost")
                port = self.config.listen.get("port", None)
                if not port:
                    raise ValueError("Listen port must be specified in config for handshake address")
                self.logger.debug(f"使用 listen 配置构造握手地址: host={host}, port={port}")
            else:
                # 否则使用 transport 配置
                host = self.config.transport.grpc.host
                port = self.config.transport.grpc.port
                self.logger.debug(f"使用 transport 配置构造握手地址: host={host}, port={port}")

            # 如果 host 是 0.0.0.0，替换为 localhost（本地开发）
            if host == "0.0.0.0":
                host = "localhost"
            handshake_address = f"{host}:{port}"

        self.logger.debug(f"发送握手通知到 {target}，告知地址: {handshake_address}")

        try:
            await self.notify(target, "_handshake", {
                "node_id": self.node_id,
                "address": handshake_address,
            })
        except Exception as e:
            self.logger.debug(f"Handshake to {target} failed (may be expected): {e}")
    
    async def disconnect(self, target: str) -> None:
        """断开与目标节点的连接"""
        await self._transport.disconnect(target)
    
    def get_connected_nodes(self) -> List[str]:
        """获取所有已连接节点的 ID"""
        return [
            node_id
            for node_id, info in self._connection_table.items()
            if info.status == ConnectionStatus.CONNECTED
        ]
    
    def is_connected(self, node_id: str) -> bool:
        """检查是否已连接到指定节点"""
        return self._is_connected(node_id)

    async def wait_for_connections(self, min_peers: int, timeout: float = 60) -> List[str]:
        """
        等待最少数量的对等节点连接

        Args:
            min_peers: 最少节点数
            timeout: 超时时间（秒）

        Returns:
            已连接的节点 ID 列表

        Raises:
            TimeoutError: 超时未达到最少节点数

        Example:
            # 等待至少 2 个节点连接
            await node.wait_for_connections(min_peers=2, timeout=120)

            # 获取连接的节点
            peers = node.get_connected_nodes()
        """
        self.logger.debug(f"等待节点连接: 需要至少 {min_peers} 个节点，超时 {timeout}秒")
        start = time.time()
        while True:
            connected = self.get_connected_nodes()
            if len(connected) >= min_peers:
                self.logger.debug(f"已达到最少连接数: {len(connected)}/{min_peers}，已连接节点: {connected}")
                return connected

            # 检查超时
            elapsed = time.time() - start
            if elapsed > timeout:
                raise TimeoutError(
                    f"Timeout waiting for connections: {len(connected)}/{min_peers} "
                    f"after {elapsed:.1f}s"
                )

            # 等待一小段时间再检查
            await asyncio.sleep(0.5)
    
    def _is_connected(self, node_id: str) -> bool:
        """内部方法：检查连接状态"""
        info = self._connection_table.get(node_id)
        return info is not None and info.status == ConnectionStatus.CONNECTED
    
    # Transport 回调
    
    async def _on_connect(self, node_id: str, address: Optional[str]) -> None:
        """连接建立回调"""
        now = time.time()

        self._connection_table[node_id] = ConnectionInfo(
            node_id=node_id,
            address=address,
            status=ConnectionStatus.CONNECTED,
            connected_at=now,
            last_active=now,
            metadata={},
        )

        self.logger.debug(f"已连接到节点 '{node_id}' (地址: {address})")

        # 触发事件
        await self._emit("connect", node_id, address)
    
    async def _on_disconnect(self, node_id: str, reason: str) -> None:
        """连接断开回调"""

        # 更新连接表
        if node_id in self._connection_table:
            self._connection_table[node_id].status = ConnectionStatus.DISCONNECTED

        # 清理该节点的 pending 请求
        failed_requests = [
            (msg_id, pending)
            for msg_id, pending in self._pending_table.items()
            if pending.target == node_id
        ]

        for msg_id, pending in failed_requests:
            if not pending.future.done():
                pending.future.set_exception(
                    NodeDisconnectedError(node_id, reason)
                )
            self._pending_table.pop(msg_id, None)

        self.logger.info(f"已断开与节点 '{node_id}' 的连接: {reason}")

        # 触发事件
        await self._emit("disconnect", node_id, reason)
    
    # ==================== 生命周期管理 ====================
    
    async def start(self) -> None:
        """
        启动节点

        Example:
            node = Node("my_node", config)
            node.register("train", handle_train)
            await node.start()
        """
        if self._running:
            raise RuntimeError("Node is already running")

        self.logger.debug(f"正在启动节点: {self.node_id}")

        # 保存主事件循环引用（用于控制线程跨线程调用）
        self._main_loop = asyncio.get_event_loop()

        # 初始化组件
        self.logger.debug("初始化序列化器注册表")
        self._serializer_registry = SerializerRegistry(self.config.serialization)
        self._interceptor_chain = InterceptorChain()

        # 获取拦截器配置（可能为 None）
        interceptors = self.config.interceptors
        
        # 添加日志拦截器（默认启用）
        if interceptors is None or getattr(interceptors, 'logging', True):
            self.logger.debug("添加日志拦截器")
            self._interceptor_chain.add(LoggingInterceptor(node_id=self.node_id))

        # 添加认证拦截器
        if interceptors is not None and getattr(interceptors, 'auth', False):
            self.logger.debug("添加认证拦截器")
            auth_config = interceptors.auth_config
            auth_interceptor = AuthInterceptor(
                token=auth_config.token,
            )
            self._interceptor_chain.add(auth_interceptor)

        # 创建 Transport
        self.logger.debug(f"创建传输层: mode={self.config.transport.mode}")
        self._transport = create_transport(self.config.transport, self)

        # 注册内置处理器
        self.logger.debug("注册内置处理器")
        self._register_builtin_handlers()

        # 启动 Transport
        self.logger.debug("启动传输层")
        await self._transport.start()

        # 启动后台任务
        self.logger.debug("启动后台任务")
        self._start_background_tasks()

        self._running = True
        self.logger.debug(f"节点 '{self.node_id}' 启动完成")

    async def _wait_for_in_flight_requests(self, timeout: float = 5.0) -> bool:
        """
        等待所有 in-flight 请求完成

        Args:
            timeout: 超时时间（秒）

        Returns:
            是否所有请求都完成（True）或超时（False）
        """
        import time
        start = time.time()

        while True:
            async with self._in_flight_lock:
                count = self._in_flight_requests

            if count == 0:
                self.logger.debug(f"所有 in-flight 请求已完成")
                return True

            elapsed = time.time() - start
            if elapsed >= timeout:
                self.logger.warning(
                    f"等待 in-flight 请求超时: "
                    f"仍有 {count} 个请求未完成"
                )
                return False

            # 每 100ms 检查一次
            await asyncio.sleep(0.1)

    async def stop(self) -> None:
        """
        停止节点（分阶段关闭）

        阶段1: 停止接受新请求
        阶段2: 等待 in-flight 请求完成
        阶段3: 停止后台任务
        阶段4: 取消 pending 请求
        阶段5: 停止 Transport（包括控制线程）
        """
        if not self._running:
            return

        self.logger.debug(f"开始停止节点...")

        # 阶段1: 停止接受新请求
        self._accepting_requests = False
        self.logger.debug(f"已停止接受新请求")

        # 阶段2: 等待 in-flight 请求完成（最多等待5秒）
        await self._wait_for_in_flight_requests(timeout=5.0)

        # 阶段3: 停止后台任务
        self._stop_background_tasks()
        self.logger.debug(f"后台任务已停止")

        # 阶段4: 取消所有 pending 请求
        for msg_id, pending in list(self._pending_table.items()):
            if not pending.future.done():
                pending.future.cancel()
        self._pending_table.clear()
        self.logger.debug(f"Pending 请求已清理")

        # 阶段5: 停止 Transport（包括控制线程）
        if self._transport:
            await self._transport.stop()
            self.logger.debug(f"Transport 已停止")

        self._running = False
        self.logger.debug(f"节点已完全停止")
    
    def _register_builtin_handlers(self) -> None:
        """注册框架内置处理器"""
        # 握手处理器
        if "_handshake" not in self._method_table:
            self.register("_handshake", self._builtin_handshake_handler)
        
        # 心跳处理器
        if "_heartbeat" not in self._method_table:
            self.register("_heartbeat", self._builtin_heartbeat_handler)
    
    async def _builtin_handshake_handler(
        self,
        payload: Dict[str, Any],
        ctx: MessageContext,
    ) -> None:
        """处理握手，自动回连"""
        source_id = payload.get("node_id")
        source_addr = payload.get("address")

        self.logger.debug(f"收到握手: from={source_id}, address={source_addr}")

        # 如果还没连接，则回连（仅 gRPC 模式）
        if (
            source_addr
            and source_id
            and not self.is_connected(source_id)
            and self.config.transport.mode == "grpc"
        ):
            self.logger.debug(f"准备自动回连到 {source_id} (地址: {source_addr})")
            try:
                await self._transport.connect(source_id, source_addr)
                self.logger.debug(f"自动回连成功: {source_id}")
            except Exception as e:
                self.logger.warning(f"自动回连失败: {source_id}, error={e}")
    
    async def _builtin_heartbeat_handler(
        self,
        payload: Dict[str, Any],
        ctx: MessageContext,
    ) -> None:
        """处理心跳，更新 last_active"""
        source = ctx.source
        if source in self._connection_table:
            self._connection_table[source].last_active = time.time()
    
    def _start_background_tasks(self) -> None:
        """启动后台任务"""
        # 超时检查任务
        task = asyncio.create_task(self._timeout_checker_loop())
        self._background_tasks.append(task)

        # 心跳任务（已迁移到 GRPCTransport 控制线程）
        # Memory 模式无需心跳，gRPC 模式由 GRPCTransport 内部处理
        # if self.config.heartbeat.enabled:
        #     task = asyncio.create_task(self._heartbeat_loop())
        #     self._background_tasks.append(task)
    
    def _stop_background_tasks(self) -> None:
        """停止后台任务"""
        for task in self._background_tasks:
            task.cancel()
        self._background_tasks.clear()
    
    async def _timeout_checker_loop(self) -> None:
        """检查超时的请求"""
        while self._running:
            try:
                await asyncio.sleep(1.0)
                
                now = time.time()
                expired = [
                    msg_id
                    for msg_id, pending in self._pending_table.items()
                    if now > pending.timeout_at and not pending.future.done()
                ]
                
                for msg_id in expired:
                    pending = self._pending_table.pop(msg_id, None)
                    if pending and not pending.future.done():
                        timeout_duration = pending.timeout_at - pending.created_at
                        pending.future.set_exception(
                            CallTimeoutError(
                                msg_id=msg_id,
                                method=pending.method,
                                target=pending.target,
                                timeout=timeout_duration,
                            )
                        )
            except asyncio.CancelledError:
                break
            except Exception as e:
                self.logger.error(f"Timeout checker error: {e}")
    
    async def _heartbeat_loop(self) -> None:
        """
        心跳循环：发送 + 检测超时

        超时处理策略（宽限期机制）：
        1. 第一次超时：进入宽限期（30秒），记录警告日志
        2. 宽限期内恢复：清除宽限期，重置计数器
        3. 宽限期结束仍超时：真正断开连接

        这样可以避免因网络波动导致的频繁断开重连。
        """
        interval = self.config.heartbeat.interval
        timeout = self.config.heartbeat.timeout
        grace_period = 30.0  # 宽限期30秒

        while self._running:
            try:
                await asyncio.sleep(interval)

                now = time.time()

                # 1. 检查所有连接
                for node_id, info in list(self._connection_table.items()):
                    if info.status != ConnectionStatus.CONNECTED:
                        continue

                    time_since_active = now - info.last_active

                    # 情况1: 连接正常（在超时时间内）
                    if time_since_active <= timeout:
                        # 如果之前在宽限期，现在恢复了，清除宽限期状态
                        if info.grace_period_end is not None:
                            self.logger.info(f"Connection recovered: {node_id} (was in grace period)")
                            info.grace_period_end = None
                            info.timeout_count = 0
                        continue

                    # 情况2: 第一次超时，进入宽限期
                    if info.grace_period_end is None:
                        info.grace_period_end = now + grace_period
                        info.timeout_count = 1
                        self.logger.warning(
                            f"Heartbeat timeout detected: {node_id} "
                            f"(silent for {time_since_active:.1f}s, timeout={timeout}s). "
                            f"Entering grace period ({grace_period}s)"
                        )
                        continue

                    # 情况3: 宽限期内，继续等待
                    if now < info.grace_period_end:
                        info.timeout_count += 1
                        self.logger.debug(
                            f"Still in grace period: {node_id} "
                            f"(timeout_count={info.timeout_count}, "
                            f"remaining={info.grace_period_end - now:.1f}s)"
                        )
                        continue

                    # 情况4: 宽限期结束仍超时，真正断开
                    self.logger.error(
                        f"Heartbeat timeout (grace period expired): {node_id} "
                        f"(silent for {time_since_active:.1f}s, timeout_count={info.timeout_count})"
                    )
                    await self._handle_connection_timeout(node_id)

                # 2. 发送心跳到所有存活连接
                connected_nodes = self.get_connected_nodes()
                if connected_nodes:
                    await self.broadcast(
                        targets=connected_nodes,
                        method="_heartbeat",
                        payload={"timestamp": now},
                    )
            except asyncio.CancelledError:
                break
            except Exception as e:
                self.logger.error(f"Heartbeat loop error: {e}")
    
    async def _handle_connection_timeout(self, node_id: str) -> None:
        """处理连接超时"""
        # 1. 标记为断开
        if node_id in self._connection_table:
            self._connection_table[node_id].status = ConnectionStatus.DISCONNECTED
        
        # 2. 关闭底层连接
        try:
            await self._transport.disconnect(node_id)
        except Exception as e:
            self.logger.debug(f"Error disconnecting {node_id}: {e}")
        
        # 3. 触发 _on_disconnect
        await self._on_disconnect(node_id, "heartbeat timeout")
    
    # ==================== 辅助方法 ====================
    
    def _get_serializer(self, method: str, serializer_name: Optional[str] = None) -> Serializer:
        """获取指定方法的序列化器

        优先级: 显式指定 > 代码注册的 MethodOptions > YAML 配置 > 默认
        """
        # 0. 显式指定
        if serializer_name:
            result = self._serializer_registry.get(serializer_name)
            self.logger.debug(f"Method '{method}': using explicit serializer '{serializer_name}'")
            return result

        # 1. 代码注册的 MethodOptions
        entry = self._method_table.get(method)
        if entry and entry.options.serializer:
            result = self._serializer_registry.get(entry.options.serializer)
            self.logger.debug(f"Method '{method}': using MethodOptions serializer '{entry.options.serializer}'")
            return result

        # 2. YAML 配置 / 默认
        result = self._serializer_registry.get_for_method(method)
        # self.logger.debug(f"Method '{method}': using default/config serializer '{result.name}' (config.default={self.config.serialization.default})")
        return result
    
    # ==================== 公开扩展 API ====================
    
    def register_serializer(self, serializer: Serializer) -> None:
        """
        注册自定义序列化器
        
        Args:
            serializer: 序列化器实例，需要实现 name, serialize(), deserialize()
            
        Example:
            class MsgPackSerializer(Serializer):
                @property
                def name(self) -> str:
                    return "msgpack"
                
                def serialize(self, data: Any) -> bytes:
                    import msgpack
                    return msgpack.packb(data)
                
                def deserialize(self, data: bytes) -> Any:
                    import msgpack
                    return msgpack.unpackb(data)
            
            node.register_serializer(MsgPackSerializer())
        """
        if self._serializer_registry is None:
            raise RuntimeError("Node not started, call start() first")
        self._serializer_registry.register(serializer)
    
    def add_interceptor(self, interceptor: Interceptor, prepend: bool = False) -> None:
        """
        添加拦截器
        
        Args:
            interceptor: 拦截器实例
            prepend: 是否添加到最外层（最先执行）
            
        执行顺序遵循"洋葱模型"：
        - 添加顺序: [A, B, C]
        - 执行顺序: C.before -> B.before -> A.before -> handler -> A.after -> B.after -> C.after
            
        Example:
            class MetricsInterceptor(Interceptor):
                async def intercept_inbound(self, message, context, next_handler):
                    start = time.time()
                    result = await next_handler(message)
                    duration = time.time() - start
                    record_metric(message.method, duration)
                    return result
                    
                async def intercept_outbound(self, message, context, next_handler):
                    return await next_handler(message)
            
            node.add_interceptor(MetricsInterceptor())
        """
        if self._interceptor_chain is None:
            raise RuntimeError("Node not started, call start() first")
        self._interceptor_chain.add(interceptor, prepend=prepend)
    
    def remove_interceptor(self, interceptor: Interceptor) -> bool:
        """
        移除拦截器
        
        Args:
            interceptor: 要移除的拦截器实例
            
        Returns:
            是否成功移除
        """
        if self._interceptor_chain is None:
            return False
        return self._interceptor_chain.remove(interceptor)
    
    # ==================== 上下文管理器 ====================
    
    async def __aenter__(self) -> "Node":
        await self.start()
        return self
    
    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        await self.stop()


# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import node_service_pb2 as node__service__pb2


class NodeServiceStub(object):
    """节点服务
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Call = channel.unary_unary(
                '/node.NodeService/Call',
                request_serializer=node__service__pb2.NodeMessage.SerializeToString,
                response_deserializer=node__service__pb2.NodeMessage.FromString,
                )
        self.Notify = channel.unary_unary(
                '/node.NodeService/Notify',
                request_serializer=node__service__pb2.NodeMessage.SerializeToString,
                response_deserializer=node__service__pb2.Empty.FromString,
                )


class NodeServiceServicer(object):
    """节点服务
    """

    def Call(self, request, context):
        """请求-响应调用
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Notify(self, request, context):
        """单向通知
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_NodeServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Call': grpc.unary_unary_rpc_method_handler(
                    servicer.Call,
                    request_deserializer=node__service__pb2.NodeMessage.FromString,
                    response_serializer=node__service__pb2.NodeMessage.SerializeToString,
            ),
            'Notify': grpc.unary_unary_rpc_method_handler(
                    servicer.Notify,
                    request_deserializer=node__service__pb2.NodeMessage.FromString,
                    response_serializer=node__service__pb2.Empty.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'node.NodeService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class NodeService(object):
    """节点服务
    """

    @staticmethod
    def Call(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/node.NodeService/Call',
            node__service__pb2.NodeMessage.SerializeToString,
            node__service__pb2.NodeMessage.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Notify(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/node.NodeService/Notify',
            node__service__pb2.NodeMessage.SerializeToString,
            node__service__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
